#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import html.parser
import ssl
from urllib import parse
from urllib import request

from tempest.common import utils
from tempest import config
from tempest.lib import decorators
from tempest import test

CONF = config.CONF


class HorizonHTMLParser(html.parser.HTMLParser):
    csrf_token = None
    region = None
    login = None

    def _find_name(self, attrs, name):
        for attrpair in attrs:
            if attrpair[0] == 'name' and attrpair[1] == name:
                return True
        return False

    def _find_value(self, attrs):
        for attrpair in attrs:
            if attrpair[0] == 'value':
                return attrpair[1]
        return None

    def _find_attr_value(self, attrs, attr_name):
        for attrpair in attrs:
            if attrpair[0] == attr_name:
                return attrpair[1]
        return None

    def handle_starttag(self, tag, attrs):
        if tag == 'input':
            if self._find_name(attrs, 'csrfmiddlewaretoken'):
                self.csrf_token = self._find_value(attrs)
            if self._find_name(attrs, 'region'):
                self.region = self._find_value(attrs)
        if tag == 'form':
            self.login = self._find_attr_value(attrs, 'action')


class TestDashboardBasicOps(test.BaseTestCase):

    """The test suite for dashboard basic operations

    This is a basic scenario test:
    * checks that the login page is available
    * logs in as a regular user
    * checks that the user home page loads without error
    """
    opener = None

    credentials = ['primary']

    @classmethod
    def skip_checks(cls):
        super(TestDashboardBasicOps, cls).skip_checks()
        if not CONF.service_available.horizon:
            raise cls.skipException("Horizon support is required")

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources()
        super(TestDashboardBasicOps, cls).setup_credentials()

    def check_login_page(self):
        response = self._get_opener().open(CONF.dashboard.dashboard_url).read()
        self.assertIn("id_username", response.decode("utf-8"))

    def user_login(self, username, password):
        response = self._get_opener().open(CONF.dashboard.dashboard_url).read()

        # Grab the CSRF token and default region
        parser = HorizonHTMLParser()
        parser.feed(response.decode("utf-8"))

        # construct login url for dashboard, discovery accommodates non-/ web
        # root for dashboard
        login_url = parse.urljoin(CONF.dashboard.dashboard_url, parser.login)

        # Prepare login form request
        req = request.Request(login_url)
        req.add_header('Content-type', 'application/x-www-form-urlencoded')
        req.add_header('Referer', CONF.dashboard.dashboard_url)

        # Pass the default domain name regardless of the auth version in order
        # to test the scenario of when horizon is running with keystone v3
        params = {'username': username,
                  'password': password,
                  'region': parser.region,
                  'domain': CONF.auth.default_credentials_domain_name,
                  'csrfmiddlewaretoken': parser.csrf_token}
        self._get_opener().open(req, parse.urlencode(params).encode())

    def check_home_page(self):
        response = self._get_opener().open(CONF.dashboard.dashboard_url).read()
        self.assertIn('Overview', response.decode("utf-8"))

    def _get_opener(self):
        if not self.opener:
            if (CONF.dashboard.disable_ssl_certificate_validation and
                    self._ssl_default_context_supported()):
                ctx = ssl.create_default_context()
                ctx.check_hostname = False
                ctx.verify_mode = ssl.CERT_NONE
                self.opener = request.build_opener(
                    request.HTTPSHandler(context=ctx),
                    request.HTTPCookieProcessor())
            else:
                self.opener = request.build_opener(
                    request.HTTPCookieProcessor())
        return self.opener

    def _ssl_default_context_supported(self):
        return (hasattr(ssl, 'create_default_context'))

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('4f8851b1-0e69-482b-b63b-84c6e76f6c80')
    @utils.services('dashboard')
    def test_basic_scenario(self):
        creds = self.os_primary.credentials
        self.check_login_page()
        self.user_login(creds.username, creds.password)
        self.check_home_page()



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest.lib.services.identity.v3.access_rules_client import \
    AccessRulesClient
from tempest.lib.services.identity.v3.application_credentials_client import \
    ApplicationCredentialsClient
from tempest.lib.services.identity.v3.catalog_client import \
    CatalogClient
from tempest.lib.services.identity.v3.credentials_client import \
    CredentialsClient
from tempest.lib.services.identity.v3.domain_configuration_client \
    import DomainConfigurationClient
from tempest.lib.services.identity.v3.domains_client import DomainsClient
from tempest.lib.services.identity.v3.endpoint_filter_client import \
    EndPointsFilterClient
from tempest.lib.services.identity.v3.endpoint_groups_client import \
    EndPointGroupsClient
from tempest.lib.services.identity.v3.endpoints_client import EndPointsClient
from tempest.lib.services.identity.v3.groups_client import GroupsClient
from tempest.lib.services.identity.v3.identity_client import IdentityClient
from tempest.lib.services.identity.v3.inherited_roles_client import \
    InheritedRolesClient
from tempest.lib.services.identity.v3.limits_client import LimitsClient
from tempest.lib.services.identity.v3.oauth_consumers_client import \
    OAUTHConsumerClient
from tempest.lib.services.identity.v3.oauth_token_client import \
    OAUTHTokenClient
from tempest.lib.services.identity.v3.policies_client import PoliciesClient
from tempest.lib.services.identity.v3.project_tags_client import \
    ProjectTagsClient
from tempest.lib.services.identity.v3.projects_client import ProjectsClient
from tempest.lib.services.identity.v3.regions_client import RegionsClient
from tempest.lib.services.identity.v3.role_assignments_client import \
    RoleAssignmentsClient
from tempest.lib.services.identity.v3.roles_client import RolesClient
from tempest.lib.services.identity.v3.services_client import ServicesClient
from tempest.lib.services.identity.v3.token_client import V3TokenClient
from tempest.lib.services.identity.v3.trusts_client import TrustsClient
from tempest.lib.services.identity.v3.users_client import UsersClient
from tempest.lib.services.identity.v3.versions_client import VersionsClient

__all__ = ['AccessRulesClient', 'ApplicationCredentialsClient',
           'CatalogClient', 'CredentialsClient', 'DomainsClient',
           'DomainConfigurationClient', 'EndPointGroupsClient',
           'EndPointsClient', 'EndPointsFilterClient',
           'GroupsClient', 'IdentityClient', 'InheritedRolesClient',
           'LimitsClient', 'OAUTHConsumerClient', 'OAUTHTokenClient',
           'PoliciesClient', 'ProjectsClient', 'ProjectTagsClient',
           'RegionsClient', 'RoleAssignmentsClient', 'RolesClient',
           'ServicesClient', 'V3TokenClient', 'TrustsClient', 'UsersClient',
           'VersionsClient']



# Copyright 2013 IBM Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from urllib import parse as urllib

from tempest.api.compute import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class InstanceUsageAuditLogNegativeTestJSON(base.BaseV2ComputeAdminTest):
    """Negative tests of instance usage audit logs"""

    @classmethod
    def setup_clients(cls):
        super(InstanceUsageAuditLogNegativeTestJSON, cls).setup_clients()
        cls.adm_client = cls.os_admin.instance_usages_audit_log_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a9d33178-d2c9-4131-ad3b-f4ca8d0308a2')
    def test_instance_usage_audit_logs_with_nonadmin_user(self):
        """Test list/show instance usage audit logs by non-admin should fail

        The instance_usage_audit_logs API just can be accessed by admin user.
        """
        self.assertRaises(lib_exc.Forbidden,
                          self.instance_usages_audit_log_client.
                          list_instance_usage_audit_logs)
        now = datetime.datetime.now()
        self.assertRaises(lib_exc.Forbidden,
                          self.instance_usages_audit_log_client.
                          show_instance_usage_audit_log,
                          urllib.quote(now.strftime("%Y-%m-%d %H:%M:%S")))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9b952047-3641-41c7-ba91-a809fc5974c8')
    def test_get_instance_usage_audit_logs_with_invalid_time(self):
        """Test showing instance usage audit logs with invalid time

        Showing instance usage audit logs with invalid time should fail.
        """
        self.assertRaises(lib_exc.BadRequest,
                          self.adm_client.show_instance_usage_audit_log,
                          "invalid_time")



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools

from tempest.lib.services.network import base


def _warning_deprecate_tenant_id(func):
    @functools.wraps(func)
    def inner(*args, **kwargs):
        _self = args[0]
        # check length of arg to know whether 'tenant_id' is passed as
        # positional arg or kwargs.
        if len(args) < 2:
            if 'tenant_id' in kwargs:
                _self.LOG.warning(
                    'positional arg name "tenant_id" is deprecated, for '
                    'removal, please start using "project_id" instead')
            elif 'project_id' in kwargs:
                # fallback to deprecated name till deprecation phase.
                kwargs['tenant_id'] = kwargs.pop('project_id')

        return func(*args, **kwargs)
    return inner


class QuotasClient(base.BaseNetworkClient):

    @_warning_deprecate_tenant_id
    def update_quotas(self, tenant_id, **kwargs):
        """Update quota for a project.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-quota-for-a-project
        """
        put_body = {'quota': kwargs}
        uri = '/quotas/%s' % tenant_id
        return self.update_resource(uri, put_body)

    @_warning_deprecate_tenant_id
    def reset_quotas(self, tenant_id):  # noqa
        # NOTE: This noqa is for passing T111 check and we cannot rename
        #       to keep backwards compatibility.
        uri = '/quotas/%s' % tenant_id
        return self.delete_resource(uri)

    @_warning_deprecate_tenant_id
    def show_quotas(self, tenant_id, **fields):
        """Show quota for a project.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-quotas-for-a-project
        """
        uri = '/quotas/%s' % tenant_id
        return self.show_resource(uri, **fields)

    def list_quotas(self, **filters):
        """List quotas for projects with non default quota values.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-quotas-for-projects-with-non-default-quota-values
        """
        uri = '/quotas'
        return self.list_resources(uri, **filters)

    @_warning_deprecate_tenant_id
    def show_default_quotas(self, tenant_id):
        """List default quotas for a project."""
        uri = '/quotas/%s/default' % tenant_id
        return self.show_resource(uri)

    @_warning_deprecate_tenant_id
    def show_quota_details(self, tenant_id):
        """Show quota details for a project."""
        uri = '/quotas/%s/details.json' % tenant_id
        return self.show_resource(uri)



# Copyright 2023 Red Hat, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.image.v2 import tasks_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestImageTaskClient(base.BaseServiceTest):
    def setUp(self):
        super(TestImageTaskClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = tasks_client.TaskClient(
            fake_auth, 'image', 'regionOne')

    def test_list_task(self):
        fake_result = {

            "first": "/v2/tasks",
            "schema": "/v2/schemas/tasks",
            "tasks": [
                {
                    "id": "08b7e1c8-3821-4f54-b3b8-d6655d178cdf",
                    "owner": "fa6c8c1600f4444281658a23ee6da8e8",
                    "schema": "/v2/schemas/task",
                    "self": "/v2/tasks/08b7e1c8-3821-4f54-b3b8-d6655d178cdf",
                    "status": "processing",
                    "type": "import"
                    },
                {
                    "id": "231c311d-3557-4e23-afc4-6d98af1419e7",
                    "owner": "fa6c8c1600f4444281658a23ee6da8e8",
                    "schema": "/v2/schemas/task",
                    "self": "/v2/tasks/231c311d-3557-4e23-afc4-6d98af1419e7",
                    "status": "processing",
                    "type": "import"
                    }
                ]
            }
        self.check_service_client_function(
            self.client.list_tasks,
            'tempest.lib.common.rest_client.RestClient.get',
            fake_result,
            mock_args=['tasks'])

    def test_create_task(self):
        fake_result = {
            "type": "import",
            "input": {
                "import_from":
                "http://download.cirros-cloud.net/0.6.1/ \
                    cirros-0.6.1-x86_64-disk.img",
                "import_from_format": "qcow2",
                "image_properties": {
                    "disk_format": "qcow2",
                    "container_format": "bare"
                }
            }
            }
        self.check_service_client_function(
            self.client.create_task,
            'tempest.lib.common.rest_client.RestClient.post',
            fake_result,
            status=201)

    def test_show_task(self):
        fake_result = {
            "task_id": "08b7e1c8-3821-4f54-b3b8-d6655d178cdf"
            }
        self.check_service_client_function(
            self.client.show_tasks,
            'tempest.lib.common.rest_client.RestClient.get',
            fake_result,
            status=200,
            task_id="e485aab9-0907-4973-921c-bb6da8a8fcf8")



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_serialization import jsonutils as json

from tempest.lib.services.volume.v3 import volume_manage_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestVolumeManageClient(base.BaseServiceTest):

    VOLUME_MANAGE_REQUEST = {
        "volume": {
            "host": "controller1@rbd#rbd",
            "name": "volume-managed",
            "availability_zone": "nova",
            "bootable": False,
            "metadata": None,
            "ref": {
                "source-name": "volume-2ce6ca46-e6c1-4fe5-8268-3a1c536fcbf3"
            },
            "volume_type": None,
            "description": "volume-manage-description"
        }
    }

    VOLUME_MANAGE_RESPONSE = {
        "volume": {
            "migration_status": None,
            "attachments": [],
            "links": [
                {
                    "href": "fake-url-1",
                    "rel": "self"
                },
                {
                    "href": "fake-url-2",
                    "rel": "bookmark"
                }
            ],
            "availability_zone": "nova",
            "encrypted": False,
            "updated_at": None,
            "replication_status": None,
            "snapshot_id": None,
            "id": "c07cd4a4-b52b-4511-a176-fbaa2011a227",
            "size": 0,
            "user_id": "142d8663efce464c89811c63e45bd82e",
            "metadata": {},
            "status": "creating",
            "description": "volume-manage-description",
            "multiattach": False,
            "source_volid": None,
            "consistencygroup_id": None,
            "name": "volume-managed",
            "bootable": "false",
            "created_at": "2017-07-11T09:14:01.000000",
            "volume_type": None
        }
    }

    def setUp(self):
        super(TestVolumeManageClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = volume_manage_client.VolumeManageClient(fake_auth,
                                                              'volume',
                                                              'regionOne')

    def _test_manage_volume(self, bytes_body=False):
        payload = json.dumps(self.VOLUME_MANAGE_REQUEST, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(volume_manage_client.json,
                               'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.manage_volume,
                'tempest.lib.common.rest_client.RestClient.post',
                self.VOLUME_MANAGE_RESPONSE,
                to_utf=bytes_body,
                status=202,
                mock_args=['os-volume-manage', payload],
                **self.VOLUME_MANAGE_REQUEST['volume'])

    def test_manage_volume_with_str_body(self):
        self._test_manage_volume()

    def test_manage_volume_with_bytes_body(self):
        self._test_manage_volume(bytes_body=True)



# Copyright 2023 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client

CHUNKSIZE = 1024 * 64  # 64kB


class TaskClient(rest_client.RestClient):
    api_version = "v2"

    def create_task(self, **kwargs):
        """Create a task.

        For a full list of available parameters, please refer to the official
        API reference:
        https://developer.openstack.org/api-ref/image/v2/#create-task
        """
        data = json.dumps(kwargs)
        resp, body = self.post('tasks', data)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_tasks(self, **kwargs):
        """List tasks.

        For a full list of available parameters, please refer to the official
        API reference:
        https://developer.openstack.org/api-ref/image/v2/#list-tasks
        """
        url = 'tasks'

        if kwargs:
            url += '?%s' % urllib.urlencode(kwargs)

        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_tasks(self, task_id):
        """Show task details.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#show-task-details
        """
        url = 'tasks/%s' % task_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from unittest import mock

from tempest.cmd import cleanup
from tempest.tests import base


class TestTempestCleanup(base.TestCase):

    def test_load_json_saved_state(self):
        # instantiate "empty" TempestCleanup
        app = mock.Mock()
        c = cleanup.TempestCleanup(app, None, 'test')
        test_saved_json = 'tempest/tests/cmd/test_saved_state_json.json'
        with open(test_saved_json, 'r') as f:
            test_saved_json_content = json.load(f)
        # test if the file is loaded without any issues/exceptions
        c.options = mock.Mock()
        c.options.init_saved_state = True
        c._load_saved_state(test_saved_json)
        self.assertEqual(c.json_data, test_saved_json_content)

    def test_load_json_resource_list(self):
        # instantiate "empty" TempestCleanup
        app = mock.Mock()
        c = cleanup.TempestCleanup(app, None, 'test')
        test_resource_list = 'tempest/tests/cmd/test_resource_list.json'
        with open(test_resource_list, 'r') as f:
            test_resource_list_content = json.load(f)
        # test if the file is loaded without any issues/exceptions
        c.options = mock.Mock()
        c.options.init_saved_state = False
        c.options.resource_list = True
        c._load_resource_list(test_resource_list)
        self.assertEqual(c.resource_data, test_resource_list_content)

    @mock.patch('tempest.cmd.cleanup.TempestCleanup.init')
    @mock.patch('tempest.cmd.cleanup.TempestCleanup._cleanup')
    def test_take_action_got_exception(self, mock_cleanup, mock_init):
        app = mock.Mock()
        c = cleanup.TempestCleanup(app, None, 'test')
        c.GOT_EXCEPTIONS.append('exception')
        mock_cleanup.return_value = True
        mock_init.return_value = True
        try:
            c.take_action(mock.Mock())
        except Exception as exc:
            self.assertEqual(str(exc), '[\'exception\']')
            return
        assert False



# Copyright 2022 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import server_external_events_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestServerExternalEventsClient(base.BaseServiceTest):

    events = [
        {
            "code": 200,
            "name": "network-changed",
            "server_uuid": "ff1df7b2-6772-45fd-9326-c0a3b05591c2",
            "status": "completed",
            "tag": "foo"
        }
    ]

    events_req = [
        {
            "name": "network-changed",
            "server_uuid": "ff1df7b2-6772-45fd-9326-c0a3b05591c2",
        }
    ]

    def setUp(self):
        super(TestServerExternalEventsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = server_external_events_client.ServerExternalEventsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_create_server_external_events(self, bytes_body=False):
        expected = {"events": self.events}
        self.check_service_client_function(
            self.client.create_server_external_events,
            'tempest.lib.common.rest_client.RestClient.post', expected,
            bytes_body, events=self.events_req)

    def test_create_server_external_events_str_body(self):
        self._test_create_server_external_events(bytes_body=False)

    def test_create_server_external_events_byte_body(self):
        self._test_create_server_external_events(bytes_body=True)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import subprocess
from unittest import mock


from tempest.lib.cli import base as cli_base
from tempest.lib import exceptions
from tempest.tests import base


class TestExecute(base.TestCase):

    @mock.patch('subprocess.Popen', autospec=True)
    def test_execute_success(self, mock_popen):
        mock_popen.return_value.returncode = 0
        mock_popen.return_value.communicate.return_value = (
            "__init__.py", "")
        result = cli_base.execute("/bin/ls", action="tempest",
                                  flags="-l -a")
        args, kwargs = mock_popen.call_args
        # Check merge_stderr == False
        self.assertEqual(subprocess.PIPE, kwargs['stderr'])
        # Check action and flags are passed
        args = args[0]
        # We just tests that all pieces are passed through, we cannot make
        # assumptions about the order
        self.assertIn("/bin/ls", args)
        self.assertIn("-l", args)
        self.assertIn("-a", args)
        self.assertIn("tempest", args)
        # The result is mocked - checking that the mock was invoked correctly
        self.assertIsInstance(result, str)
        self.assertIn("__init__.py", result)

    @mock.patch('subprocess.Popen', autospec=True)
    def test_execute_failure(self, mock_popen):
        mock_popen.return_value.returncode = 1
        mock_popen.return_value.communicate.return_value = (
            "No such option --foobar", "")
        result = cli_base.execute("/bin/ls", action="tempest.lib",
                                  flags="--foobar", merge_stderr=True,
                                  fail_ok=True)
        args, kwargs = mock_popen.call_args
        # Check the merge_stderr
        self.assertEqual(subprocess.STDOUT, kwargs['stderr'])
        # Check action and flags are passed
        args = args[0]
        # We just tests that all pieces are passed through, we cannot make
        # assumptions about the order
        self.assertIn("/bin/ls", args)
        self.assertIn("--foobar", args)
        self.assertIn("tempest.lib", args)
        # The result is mocked - checking that the mock was invoked correctly
        self.assertIsInstance(result, str)
        self.assertIn("--foobar", result)

    @mock.patch('subprocess.Popen', autospec=True)
    def test_execute_failure_raise_exception(self, mock_popen):
        mock_popen.return_value.returncode = 1
        mock_popen.return_value.communicate.return_value = (
            "No such option --foobar", "")
        self.assertRaises(exceptions.CommandFailed, cli_base.execute,
                          "/bin/ls", action="tempest", flags="--foobar",
                          merge_stderr=True)

    def test_execute_with_prefix(self):
        result = cli_base.execute("env", action="",
                                  prefix="env NEW_VAR=1")
        self.assertIsInstance(result, str)
        self.assertIn("NEW_VAR=1", result)


class TestCLIClient(base.TestCase):

    @mock.patch.object(cli_base, 'execute')
    def test_execute_with_prefix(self, mock_execute):
        cli = cli_base.CLIClient(prefix='env LAC_ALL=C')
        cli.glance('action')
        self.assertEqual(mock_execute.call_count, 1)
        self.assertEqual(mock_execute.call_args[1],
                         {'prefix': 'env LAC_ALL=C'})

    @mock.patch.object(cli_base, 'execute')
    def test_execute_with_domain_name(self, mock_execute):
        cli = cli_base.CLIClient(
            user_domain_name='default',
            project_domain_name='default'
        )
        cli.glance('action')
        self.assertEqual(mock_execute.call_count, 1)
        self.assertIn('--os-user-domain-name default',
                      mock_execute.call_args[0][2])
        self.assertIn('--os-project-domain-name default',
                      mock_execute.call_args[0][2])
        self.assertNotIn('--os-user-domain-id',
                         mock_execute.call_args[0][2])
        self.assertNotIn('--os-project-domain-id',
                         mock_execute.call_args[0][2])

    @mock.patch.object(cli_base, 'execute')
    def test_execute_with_domain_id(self, mock_execute):
        cli = cli_base.CLIClient(
            user_domain_id='default',
            project_domain_id='default'
        )
        cli.glance('action')
        self.assertEqual(mock_execute.call_count, 1)
        self.assertIn('--os-user-domain-id default',
                      mock_execute.call_args[0][2])
        self.assertIn('--os-project-domain-id default',
                      mock_execute.call_args[0][2])
        self.assertNotIn('--os-user-domain-name',
                         mock_execute.call_args[0][2])
        self.assertNotIn('--os-project-domain-name',
                         mock_execute.call_args[0][2])

    @mock.patch.object(cli_base, 'execute')
    def test_execute_with_default_api_version(self, mock_execute):
        cli = cli_base.CLIClient()
        cli.openstack('action')
        self.assertEqual(mock_execute.call_count, 1)
        self.assertNotIn('--os-identity-api-version ',
                         mock_execute.call_args[0][2])

    @mock.patch.object(cli_base, 'execute')
    def test_execute_with_empty_api_version(self, mock_execute):
        cli = cli_base.CLIClient(identity_api_version='')
        cli.openstack('action')
        self.assertEqual(mock_execute.call_count, 1)
        self.assertNotIn('--os-identity-api-version ',
                         mock_execute.call_args[0][2])

    @mock.patch.object(cli_base, 'execute')
    def test_execute_with_explicit_api_version(self, mock_execute):
        cli = cli_base.CLIClient(identity_api_version='0.0')
        cli.openstack('action')
        self.assertEqual(mock_execute.call_count, 1)
        self.assertIn('--os-identity-api-version 0.0 ',
                      mock_execute.call_args[0][2])



# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class ServicesClient(rest_client.RestClient):
    api_version = "v2.0"

    def create_service(self, **kwargs):
        """Create a service.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/#create-service-admin-extension
        """
        post_body = json.dumps({'OS-KSADM:service': kwargs})
        resp, body = self.post('/OS-KSADM/services', post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_service(self, service_id):
        """Get Service."""
        url = '/OS-KSADM/services/%s' % service_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_services(self, **params):
        """List Service - Returns Services.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-ext/#list-services-admin-extension
        """
        url = '/OS-KSADM/services'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_service(self, service_id):
        """Delete Service."""
        url = '/OS-KSADM/services/%s' % service_id
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.services.volume.v3 import volumes_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestVolumesClient(base.BaseServiceTest):

    FAKE_VOLUME_SUMMARY = {
        "volume-summary": {
            "total_size": 4,
            "total_count": 4,
        }
    }

    FAKE_VOLUME_METADATA_ITEM = {
        "meta": {
            "name": "metadata1"
        }
    }

    FAKE_VOLUME_IMAGE_METADATA = {
        "metadata": {
            "key1": "value1",
            "key2": "value2"
        }
    }

    def setUp(self):
        super(TestVolumesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = volumes_client.VolumesClient(fake_auth,
                                                   'volume',
                                                   'regionOne')

    def _test_retype_volume(self, bytes_body=False):
        kwargs = {
            "new_type": "dedup-tier-replication",
            "migration_policy": "never"
        }

        self.check_service_client_function(
            self.client.retype_volume,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            to_utf=bytes_body,
            status=202,
            volume_id="a3be971b-8de5-4bdf-bdb8-3d8eb0fb69f8",
            **kwargs
        )

    def _test_force_detach_volume(self, bytes_body=False):
        kwargs = {
            'attachment_id': '6980e295-920f-412e-b189-05c50d605acd',
            'connector': {
                'initiator': 'iqn.2017-04.org.fake:01'
            }
        }

        self.check_service_client_function(
            self.client.force_detach_volume,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            to_utf=bytes_body,
            status=202,
            volume_id="a3be971b-8de5-4bdf-bdb8-3d8eb0fb69f8",
            **kwargs
        )

    def _test_show_volume_metadata_item(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_volume_metadata_item,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_VOLUME_METADATA_ITEM,
            to_utf=bytes_body,
            volume_id="a3be971b-8de5-4bdf-bdb8-3d8eb0fb69f8",
            id="key1")

    def _test_show_volume_image_metadata(self, bytes_body=False):
        fake_volume_id = "a3be971b-8de5-4bdf-bdb8-3d8eb0fb69f8"
        self.check_service_client_function(
            self.client.show_volume_image_metadata,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_VOLUME_IMAGE_METADATA,
            to_utf=bytes_body,
            mock_args=['volumes/%s/action' % fake_volume_id,
                       json.dumps({"os-show_image_metadata": {}})],
            volume_id=fake_volume_id)

    def _test_show_volume_summary(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_volume_summary,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_VOLUME_SUMMARY,
            bytes_body)

    def test_force_detach_volume_with_str_body(self):
        self._test_force_detach_volume()

    def test_force_detach_volume_with_bytes_body(self):
        self._test_force_detach_volume(bytes_body=True)

    def test_show_volume_metadata_item_with_str_body(self):
        self._test_show_volume_metadata_item()

    def test_show_volume_metadata_item_with_bytes_body(self):
        self._test_show_volume_metadata_item(bytes_body=True)

    def test_show_volume_image_metadata_with_str_body(self):
        self._test_show_volume_image_metadata()

    def test_show_volume_image_metadata_with_bytes_body(self):
        self._test_show_volume_image_metadata(bytes_body=True)

    def test_retype_volume_with_str_body(self):
        self._test_retype_volume()

    def test_retype_volume_with_bytes_body(self):
        self._test_retype_volume(bytes_body=True)

    def test_show_volume_summary_with_str_body(self):
        self._test_show_volume_summary()

    def test_show_volume_summary_with_bytes_body(self):
        self._test_show_volume_summary(bytes_body=True)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ipaddress

import netaddr
import testtools

from tempest.api.network import base_security_groups as sec_base
from tempest.common import custom_matchers
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class PortsTestJSON(sec_base.BaseSecGroupTest):
    """Test the following operations for ports:

        port create
        port delete
        port list
        port show
        port update
    """

    @classmethod
    def resource_setup(cls):
        super(PortsTestJSON, cls).resource_setup()
        cls.network = cls.create_network()
        cls.port = cls.create_port(cls.network)

    def _delete_port(self, port_id):
        self.ports_client.delete_port(port_id)
        body = self.ports_client.list_ports()
        ports_list = body['ports']
        self.assertFalse(port_id in [n['id'] for n in ports_list])

    def _create_subnet(self, network, gateway='',
                       cidr=None, mask_bits=None, **kwargs):
        subnet = self.create_subnet(
            network, gateway, cidr, mask_bits, **kwargs)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.subnets_client.delete_subnet, subnet['id'])
        return subnet

    def _create_network(self, network_name=None, **kwargs):
        network_name = network_name or data_utils.rand_name(
            self.__class__.__name__, prefix=CONF.resource_name_prefix)
        network = self.networks_client.create_network(
            name=network_name, **kwargs)['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.networks_client.delete_network,
                        network['id'])
        return network

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c72c1c0c-2193-4aca-aaa4-b1442640f51c')
    def test_create_update_delete_port(self):
        """Test creating, updating and deleting port"""
        # Verify port creation
        body = self.ports_client.create_port(
            network_id=self.network['id'],
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix))
        port = body['port']
        # Schedule port deletion with verification upon test completion
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port['id'])
        self.addCleanup(self._delete_port, port['id'])
        self.assertTrue(port['admin_state_up'])
        # Verify port update
        new_name = "New_Port"
        body = self.ports_client.update_port(port['id'],
                                             name=new_name,
                                             admin_state_up=False)
        updated_port = body['port']
        self.assertEqual(updated_port['name'], new_name)
        self.assertFalse(updated_port['admin_state_up'])

    @decorators.idempotent_id('67f1b811-f8db-43e2-86bd-72c074d4a42c')
    def test_create_bulk_port(self):
        """Test creating multiple ports in a single request"""
        network1 = self.network
        network2 = self._create_network()
        network_list = [network1['id'], network2['id']]
        port_list = [{'network_id': net_id} for net_id in network_list]
        body = self.ports_client.create_bulk_ports(ports=port_list)
        created_ports = body['ports']
        port1 = created_ports[0]
        port2 = created_ports[1]
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port1['id'])
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port2['id'])
        self.addCleanup(self._delete_port, port1['id'])
        self.addCleanup(self._delete_port, port2['id'])
        self.assertEqual(port1['network_id'], network1['id'])
        self.assertEqual(port2['network_id'], network2['id'])
        self.assertTrue(port1['admin_state_up'])
        self.assertTrue(port2['admin_state_up'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('0435f278-40ae-48cb-a404-b8a087bc09b1')
    def test_create_port_in_allowed_allocation_pools(self):
        """Test creating port in allowed allocation pools"""
        network = self._create_network()
        net_id = network['id']
        address = self.cidr
        address.prefixlen = self.mask_bits
        if ((address.version == 4 and address.prefixlen >= 30) or
            (address.version == 6 and address.prefixlen >= 126)):
            msg = ("Subnet %s isn't large enough for the test" % address.cidr)
            raise exceptions.InvalidConfiguration(msg)
        allocation_pools = {'allocation_pools': [{'start': str(address[2]),
                                                  'end': str(address[-2])}]}
        self._create_subnet(network, cidr=address,
                            mask_bits=address.prefixlen,
                            **allocation_pools)
        body = self.ports_client.create_port(
            network_id=net_id,
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix))
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        body['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, body['port']['id'])
        port = body['port']
        ip_address = port['fixed_ips'][0]['ip_address']
        start_ip_address = allocation_pools['allocation_pools'][0]['start']
        end_ip_address = allocation_pools['allocation_pools'][0]['end']
        ip_range = netaddr.IPRange(start_ip_address, end_ip_address)
        self.assertIn(ip_address, ip_range)
        self.ports_client.delete_port(port['id'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c9a685bd-e83f-499c-939f-9f7863ca259f')
    def test_show_port(self):
        """Verify the details of port"""
        body = self.ports_client.show_port(self.port['id'])
        port = body['port']
        self.assertIn('id', port)
        # NOTE(rfolco): created_at and updated_at may get inconsistent values
        # due to possible delay between POST request and resource creation.
        # TODO(rfolco): Neutron Bug #1365341 is fixed, can remove the key
        # extra_dhcp_opts in the O release (K/L gate jobs still need it).
        self.assertThat(self.port,
                        custom_matchers.MatchesDictExceptForKeys
                        (port, excluded_keys=['extra_dhcp_opts',
                                              'created_at',
                                              'updated_at']))

    @decorators.idempotent_id('45fcdaf2-dab0-4c13-ac6c-fcddfb579dbd')
    def test_show_port_fields(self):
        """Verify specific fields of a port"""
        fields = ['id', 'mac_address']
        body = self.ports_client.show_port(self.port['id'],
                                           fields=fields)
        port = body['port']
        self.assertEqual(sorted(port.keys()), sorted(fields))
        for field_name in fields:
            self.assertEqual(port[field_name], self.port[field_name])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('cf95b358-3e92-4a29-a148-52445e1ac50e')
    def test_list_ports(self):
        """Verify the port exists in the list of all ports"""
        body = self.ports_client.list_ports()
        ports = [port['id'] for port in body['ports']
                 if port['id'] == self.port['id']]
        self.assertNotEmpty(ports, "Created port not found in the list")

    @decorators.idempotent_id('e7fe260b-1e79-4dd3-86d9-bec6a7959fc5')
    def test_port_list_filter_by_ip(self):
        """Test listing ports filtered by ip"""
        # Create network and subnet
        network = self._create_network()
        self._create_subnet(network)
        # Create two ports
        port_1 = self.ports_client.create_port(
            network_id=network['id'],
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix))
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port_1['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port_1['port']['id'])
        port_2 = self.ports_client.create_port(
            network_id=network['id'],
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix))
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port_2['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port_2['port']['id'])
        # List ports filtered by fixed_ips
        port_1_fixed_ip = port_1['port']['fixed_ips'][0]['ip_address']
        fixed_ips = 'ip_address=' + port_1_fixed_ip
        port_list = self.ports_client.list_ports(fixed_ips=fixed_ips)
        # Check that we got the desired port
        ports = port_list['ports']
        project_ids = set([port['project_id'] for port in ports])
        self.assertEqual(len(project_ids), 1,
                         'Ports from multiple projects are in the list resp')
        port_ids = [port['id'] for port in ports]
        fixed_ips = [port['fixed_ips'] for port in ports]
        port_net_ids = [port['network_id'] for port in ports]
        self.assertIn(port_1['port']['id'], port_ids)
        self.assertIn(network['id'], port_net_ids)
        # Check that every port has a fixed_ip that matches the query
        for addr in fixed_ips:
            port_ips = [port['ip_address'] for port in addr]
            self.assertIn(port_1_fixed_ip, port_ips,
                          'Port not matching IP filter found')

    @decorators.idempotent_id('79895408-85d5-460d-94e7-9531c5fd9123')
    @testtools.skipUnless(
        utils.is_extension_enabled('ip-substring-filtering', 'network'),
        'ip-substring-filtering extension not enabled.')
    def test_port_list_filter_by_ip_substr(self):
        """Test listing ports filtered by part of ip address string"""
        # Create network and subnet
        network = self._create_network()
        subnet = self._create_subnet(network)
        # Get two IP addresses
        ip_address_1 = None
        ip_address_2 = None
        ip_network = ipaddress.ip_network(str(subnet['cidr']))
        for ip in ip_network:
            if ip == ip_network.network_address:
                continue
            if ip_address_1 is None:
                ip_address_1 = str(ip)
            else:
                ip_address_2 = ip_address_1
                ip_address_1 = str(ip)
                # Make sure these two IP addresses have different substring
                if ip_address_1[:-1] != ip_address_2[:-1]:
                    break

        # Create two ports
        fixed_ips = [{'subnet_id': subnet['id'], 'ip_address': ip_address_1}]
        port_1 = self.ports_client.create_port(
            network_id=network['id'],
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix),
            fixed_ips=fixed_ips)
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port_1['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port_1['port']['id'])
        fixed_ips = [{'subnet_id': subnet['id'], 'ip_address': ip_address_2}]
        port_2 = self.ports_client.create_port(
            network_id=network['id'],
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix),
            fixed_ips=fixed_ips)
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port_2['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port_2['port']['id'])

        # Scenario 1: List port1 (port2 is filtered out)
        if ip_address_1[:-1] != ip_address_2[:-1]:
            ips_filter = 'ip_address_substr=' + ip_address_1[:-1]
        else:
            ips_filter = 'ip_address_substr=' + ip_address_1
        ports = self.ports_client.list_ports(fixed_ips=ips_filter)['ports']
        # Check that we got the desired port
        port_ids = [port['id'] for port in ports]
        fixed_ips = [port['fixed_ips'] for port in ports]
        port_ips = []
        for addr in fixed_ips:
            port_ips.extend([a['ip_address'] for a in addr])

        port_net_ids = [port['network_id'] for port in ports]
        self.assertIn(network['id'], port_net_ids)
        self.assertIn(port_1['port']['id'], port_ids)
        self.assertIn(port_1['port']['fixed_ips'][0]['ip_address'], port_ips)
        self.assertNotIn(port_2['port']['id'], port_ids)
        self.assertNotIn(
            port_2['port']['fixed_ips'][0]['ip_address'], port_ips)

        # Scenario 2: List both port1 and port2
        substr = ip_address_1
        while substr not in ip_address_2:
            substr = substr[:-1]
        ips_filter = 'ip_address_substr=' + substr
        ports = self.ports_client.list_ports(fixed_ips=ips_filter)['ports']
        # Check that we got both port
        port_ids = [port['id'] for port in ports]
        fixed_ips = [port['fixed_ips'] for port in ports]
        port_ips = []
        for addr in fixed_ips:
            port_ips.extend([a['ip_address'] for a in addr])

        port_net_ids = [port['network_id'] for port in ports]
        self.assertIn(network['id'], port_net_ids)
        self.assertIn(port_1['port']['id'], port_ids)
        self.assertIn(port_1['port']['fixed_ips'][0]['ip_address'], port_ips)
        self.assertIn(port_2['port']['id'], port_ids)
        self.assertIn(port_2['port']['fixed_ips'][0]['ip_address'], port_ips)

    @decorators.idempotent_id('5ad01ed0-0e6e-4c5d-8194-232801b15c72')
    def test_port_list_filter_by_router_id(self):
        """Test listing ports filtered by router id"""
        # Create a router
        network = self._create_network()
        self._create_subnet(network)
        router = self.create_router()
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.routers_client.delete_router, router['id'])
        port = self.ports_client.create_port(
            network_id=network['id'],
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix))
        # Add router interface to port created above
        self.routers_client.add_router_interface(router['id'],
                                                 port_id=port['port']['id'])
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.routers_client.remove_router_interface,
                        router['id'], port_id=port['port']['id'])
        # List ports filtered by router_id
        port_list = self.ports_client.list_ports(device_id=router['id'])
        ports = port_list['ports']
        self.assertEqual(len(ports), 1)
        self.assertEqual(ports[0]['id'], port['port']['id'])
        self.assertEqual(ports[0]['device_id'], router['id'])

    @decorators.idempotent_id('ff7f117f-f034-4e0e-abff-ccef05c454b4')
    def test_list_ports_fields(self):
        """Verify specific fields of ports"""
        fields = ['id', 'mac_address']
        body = self.ports_client.list_ports(fields=fields)
        ports = body['ports']
        self.assertNotEmpty(ports, "Port list returned is empty")
        # Asserting the fields returned are correct
        for port in ports:
            self.assertEqual(sorted(fields), sorted(port.keys()))

    @decorators.idempotent_id('63aeadd4-3b49-427f-a3b1-19ca81f06270')
    def test_create_update_port_with_second_ip(self):
        """Test updating port from 2 fixed ips to 1 fixed ip and vice versa"""
        # Create a network with two subnets
        network = self._create_network()
        subnet_1 = self._create_subnet(network)
        subnet_2 = self._create_subnet(network)
        fixed_ip_1 = [{'subnet_id': subnet_1['id']}]
        fixed_ip_2 = [{'subnet_id': subnet_2['id']}]

        fixed_ips = fixed_ip_1 + fixed_ip_2

        # Create a port with multiple IP addresses
        port = self.create_port(network,
                                fixed_ips=fixed_ips)
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port['id'])
        self.assertEqual(2, len(port['fixed_ips']))
        check_fixed_ips = [subnet_1['id'], subnet_2['id']]
        for item in port['fixed_ips']:
            self.assertIn(item['subnet_id'], check_fixed_ips)

        # Update the port to return to a single IP address
        port = self.update_port(port, fixed_ips=fixed_ip_1)
        self.assertEqual(1, len(port['fixed_ips']))

        # Update the port with a second IP address from second subnet
        port = self.update_port(port, fixed_ips=fixed_ips)
        self.assertEqual(2, len(port['fixed_ips']))

    def _update_port_with_security_groups(self, security_groups_names):
        subnet_1 = self._create_subnet(self.network)
        fixed_ip_1 = [{'subnet_id': subnet_1['id']}]

        security_groups_list = list()
        sec_grps_client = self.security_groups_client
        for name in security_groups_names:
            group_create_body = sec_grps_client.create_security_group(
                name=name)
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.security_groups_client.delete_security_group,
                            group_create_body['security_group']['id'])
            security_groups_list.append(group_create_body['security_group']
                                        ['id'])
        # Create a port
        sec_grp_name = data_utils.rand_name(
            name='secgroup', prefix=CONF.resource_name_prefix)
        security_group = sec_grps_client.create_security_group(
            name=sec_grp_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.security_groups_client.delete_security_group,
                        security_group['security_group']['id'])
        post_body = {
            "name": data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix),
            "security_groups": [security_group['security_group']['id']],
            "network_id": self.network['id'],
            "admin_state_up": True,
            "fixed_ips": fixed_ip_1}
        body = self.ports_client.create_port(**post_body)
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        body['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, body['port']['id'])
        port = body['port']

        # Update the port with security groups
        subnet_2 = self.create_subnet(self.network)
        fixed_ip_2 = [{'subnet_id': subnet_2['id']}]
        update_body = {
            "name": data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix),
            "admin_state_up": False,
            "fixed_ips": fixed_ip_2,
            "security_groups": security_groups_list}
        body = self.ports_client.update_port(port['id'], **update_body)
        port_show = body['port']
        # Verify the security groups and other attributes updated to port
        exclude_keys = set(port_show).symmetric_difference(update_body)
        exclude_keys.add('fixed_ips')
        exclude_keys.add('security_groups')
        self.assertThat(port_show, custom_matchers.MatchesDictExceptForKeys(
                        update_body, exclude_keys))
        self.assertEqual(fixed_ip_2[0]['subnet_id'],
                         port_show['fixed_ips'][0]['subnet_id'])

        for security_group in security_groups_list:
            self.assertIn(security_group, port_show['security_groups'])

    @decorators.idempotent_id('58091b66-4ff4-4cc1-a549-05d60c7acd1a')
    @testtools.skipUnless(
        utils.is_extension_enabled('security-group', 'network'),
        'security-group extension not enabled.')
    def test_update_port_with_security_group_and_extra_attributes(self):
        """Test updating port's security_group along with extra attributes

        First we create a port with one security group, and then we update the
        port's security_group, in the same update request we also change
        the port's fixed ips.
        """
        self._update_port_with_security_groups(
            [data_utils.rand_name('secgroup',
                                  prefix=CONF.resource_name_prefix)])

    @decorators.idempotent_id('edf6766d-3d40-4621-bc6e-2521a44c257d')
    @testtools.skipUnless(
        utils.is_extension_enabled('security-group', 'network'),
        'security-group extension not enabled.')
    def test_update_port_with_two_security_groups_and_extra_attributes(self):
        """Test updating port with two security_groups and extra attributes

        First we create a port with one security group, and then we update the
        port to two security_groups, in the same update request we also change
        the port's fixed ips.
        """
        self._update_port_with_security_groups(
            [data_utils.rand_name('secgroup',
                                  prefix=CONF.resource_name_prefix),
             data_utils.rand_name('secgroup',
                                  prefix=CONF.resource_name_prefix)])

    @decorators.idempotent_id('13e95171-6cbd-489c-9d7c-3f9c58215c18')
    def test_create_show_delete_port_user_defined_mac(self):
        """Test creating port with user defined mac address"""
        # Create a port for a legal mac
        body = self.ports_client.create_port(
            network_id=self.network['id'],
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix))
        old_port = body['port']
        free_mac_address = old_port['mac_address']
        self.ports_client.delete_port(old_port['id'])
        # Create a new port with user defined mac
        body = self.ports_client.create_port(
            network_id=self.network['id'],
            mac_address=free_mac_address,
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix))
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        body['port']['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, body['port']['id'])
        port = body['port']
        body = self.ports_client.show_port(port['id'])
        show_port = body['port']
        self.assertEqual(free_mac_address,
                         show_port['mac_address'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('4179dcb9-1382-4ced-84fe-1b91c54f5735')
    @testtools.skipUnless(
        utils.is_extension_enabled('security-group', 'network'),
        'security-group extension not enabled.')
    def test_create_port_with_no_securitygroups(self):
        """Test creating port without security groups"""
        network = self._create_network()
        self._create_subnet(network)
        port = self.create_port(network, security_groups=[])
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port['id'])
        self.assertIsNotNone(port['security_groups'])
        self.assertEmpty(port['security_groups'])


class PortsIpV6TestJSON(PortsTestJSON):
    _ip_version = 6



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.object_storage import base
from tempest.common import custom_matchers
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class StaticWebTest(base.BaseObjectTest):
    """Test static web"""

    @classmethod
    def resource_setup(cls):
        super(StaticWebTest, cls).resource_setup()

        # This header should be posted on the container before every test
        headers_public_read_acl = {'Read': '.r:*,.rlistings'}

        # Create test container and create one object in it
        cls.container_name = cls.create_container()
        cls.object_name, cls.object_data = cls.create_object(
            cls.container_name)

        cls.container_client.create_update_or_delete_container_metadata(
            cls.container_name,
            create_update_metadata=headers_public_read_acl,
            create_update_metadata_prefix="X-Container-")

    @classmethod
    def resource_cleanup(cls):
        cls.delete_containers()
        super(StaticWebTest, cls).resource_cleanup()

    @decorators.idempotent_id('c1f055ab-621d-4a6a-831f-846fcb578b8b')
    @utils.requires_ext(extension='staticweb', service='object')
    def test_web_index(self):
        """Test web index"""
        headers = {'web-index': self.object_name}

        self.container_client.create_update_or_delete_container_metadata(
            self.container_name, create_update_metadata=headers)

        # Maintain original headers, no auth added
        self.account_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=None
        )

        # test GET on http://account_url/container_name
        # we should retrieve the self.object_name file
        resp, body = self.account_client.request("GET",
                                                 self.container_name,
                                                 headers={})
        # This request is equivalent to GET object
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, self.object_data)

        # clean up before exiting
        self.container_client.create_update_or_delete_container_metadata(
            self.container_name,
            create_update_metadata={'web-index': ""})

        _, body = self.container_client.list_container_metadata(
            self.container_name)
        self.assertNotIn('x-container-meta-web-index', body)

    @decorators.idempotent_id('941814cf-db9e-4b21-8112-2b6d0af10ee5')
    @utils.requires_ext(extension='staticweb', service='object')
    def test_web_listing(self):
        """Test web listing"""
        headers = {'web-listings': 'true'}

        self.container_client.create_update_or_delete_container_metadata(
            self.container_name, create_update_metadata=headers)

        # test GET on http://account_url/container_name
        # we should retrieve a listing of objects
        resp, body = self.account_client.request("GET",
                                                 self.container_name,
                                                 headers={})
        # The target of the request is not any Swift resource. Therefore, the
        # existence of response header is checked without a custom matcher.
        self.assertIn('content-length', resp)
        self.assertIn('content-type', resp)
        self.assertIn('x-trans-id', resp)
        self.assertIn('date', resp)
        # Check only the format of common headers with custom matcher
        self.assertThat(resp, custom_matchers.AreAllWellFormatted())

        self.assertIn(self.object_name, body.decode())

        # clean up before exiting
        self.container_client.create_update_or_delete_container_metadata(
            self.container_name,
            create_update_metadata={'web-listings': ""})
        _, body = self.container_client.list_container_metadata(
            self.container_name)
        self.assertNotIn('x-container-meta-web-listings', body)

    @decorators.idempotent_id('bc37ec94-43c8-4990-842e-0e5e02fc8926')
    @utils.requires_ext(extension='staticweb', service='object')
    def test_web_listing_css(self):
        """Test web listing css"""
        headers = {'web-listings': 'true',
                   'web-listings-css': 'listings.css'}

        self.container_client.create_update_or_delete_container_metadata(
            self.container_name, create_update_metadata=headers)

        # Maintain original headers, no auth added
        self.account_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=None
        )

        # test GET on http://account_url/container_name
        # we should retrieve a listing of objects
        _, body = self.account_client.request("GET", self.container_name,
                                              headers={})
        self.assertIn(self.object_name, body.decode())
        css = '<link rel="stylesheet" type="text/css" href="listings.css" />'
        self.assertIn(css, body.decode())

    @decorators.idempotent_id('f18b4bef-212e-45e7-b3ca-59af3a465f82')
    @utils.requires_ext(extension='staticweb', service='object')
    def test_web_error(self):
        """Test web error"""
        headers = {'web-listings': 'true',
                   'web-error': self.object_name}

        self.container_client.create_update_or_delete_container_metadata(
            self.container_name, create_update_metadata=headers)

        # Create object to return when requested object not found
        object_name_404 = "404" + self.object_name
        object_data_404 = data_utils.arbitrary_string()
        self.object_client.create_object(self.container_name,
                                         object_name_404,
                                         object_data_404)

        # Do not set auth in HTTP headers for next request
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=None
        )

        # Request non-existing object
        self.assertRaises(
            lib_exc.NotFound, self.object_client.get_object,
            self.container_name, "notexisting")



# Copyright (C) 2017 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class GroupsTest(base.BaseVolumeAdminTest):
    """Tests of volume groups with microversion greater than 3.12"""

    volume_min_microversion = '3.13'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('4b111d28-b73d-4908-9bd2-03dc2992e4d4')
    def test_group_create_show_list_delete(self):
        """Test creating, showing, listing and deleting of volume group"""
        # Create volume type
        volume_type = self.create_volume_type()

        # Create group type
        group_type = self.create_group_type()

        # Create group
        grp1_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='Group1')
        grp1 = self.create_group(group_type=group_type['id'],
                                 volume_types=[volume_type['id']],
                                 name=grp1_name)
        grp1_id = grp1['id']

        grp2_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='Group2')
        grp2 = self.create_group(group_type=group_type['id'],
                                 volume_types=[volume_type['id']],
                                 name=grp2_name)
        grp2_id = grp2['id']

        # Create volume
        vol1_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name="volume")
        params = {'name': vol1_name,
                  'volume_type': volume_type['id'],
                  'group_id': grp1['id'],
                  'size': CONF.volume.volume_size}
        vol1 = self.volumes_client.create_volume(**params)['volume']
        self.assertEqual(grp1['id'], vol1['group_id'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client, vol1['id'], 'available')
        vol1_id = vol1['id']

        # Get a given group
        grp1 = self.groups_client.show_group(grp1['id'])['group']
        self.assertEqual(grp1_name, grp1['name'])
        self.assertEqual(grp1_id, grp1['id'])

        grp2 = self.groups_client.show_group(grp2['id'])['group']
        self.assertEqual(grp2_name, grp2['name'])
        self.assertEqual(grp2_id, grp2['id'])

        # Get all groups with detail
        grps = self.groups_client.list_groups(detail=True)['groups']
        for grp_id in [grp1_id, grp2_id]:
            filtered_grps = [g for g in grps if g['id'] == grp_id]
            self.assertEqual(1, len(filtered_grps))
            self.assertEqual([volume_type['id']],
                             filtered_grps[0]['volume_types'])
            self.assertEqual(group_type['id'],
                             filtered_grps[0]['group_type'])

        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        filtered_vols = [v for v in vols if v['id'] in [vol1_id]]
        self.assertEqual(1, len(filtered_vols))
        for vol in filtered_vols:
            self.assertEqual(grp1_id, vol['group_id'])

        # Delete group
        # grp1 has a volume so delete_volumes flag is set to True by default
        self.delete_group(grp1_id)
        # grp2 is empty so delete_volumes flag can be set to False
        self.delete_group(grp2_id, delete_volumes=False)
        grps = self.groups_client.list_groups(detail=True)['groups']
        self.assertEmpty(grps)

    @decorators.idempotent_id('4a8a6fd2-8b3b-4641-8f54-6a6f99320006')
    def test_group_update(self):
        """Test updating volume group"""
        # Create volume type
        volume_type = self.create_volume_type()

        # Create group type
        group_type = self.create_group_type()

        # Create Group
        grp = self.create_group(group_type=group_type['id'],
                                volume_types=[volume_type['id']])

        prefix = CONF.resource_name_prefix
        # Create volume is instance level, can not be deleted before group.
        # Volume delete handled by delete_group method, cleanup method.
        grp_vols = []
        for _ in range(2):
            params = {'name': data_utils.rand_name(prefix=prefix,
                                                   name="volume"),
                      'volume_type': volume_type['id'],
                      'group_id': grp['id'],
                      'size': CONF.volume.volume_size}
            vol = self.volumes_client.create_volume(**params)['volume']
            waiters.wait_for_volume_resource_status(
                self.volumes_client, vol['id'], 'available')
            grp_vols.append(vol)
        vol2 = grp_vols[1]

        # Remove a volume from group and update name and description
        new_grp_name = 'new_group'
        new_desc = 'This is a new group'
        grp_params = {'name': new_grp_name,
                      'description': new_desc,
                      'remove_volumes': vol2['id']}
        self.groups_client.update_group(grp['id'], **grp_params)

        # Wait for group status to become available
        waiters.wait_for_volume_resource_status(
            self.groups_client, grp['id'], 'available')

        # Get the updated Group
        grp = self.groups_client.show_group(grp['id'])['group']
        self.assertEqual(new_grp_name, grp['name'])
        self.assertEqual(new_desc, grp['description'])

        # Get volumes in the group
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        grp_vols = [v for v in vols if v['group_id'] == grp['id']]
        self.assertEqual(1, len(grp_vols))

        # Add a volume to the group
        grp_params = {'add_volumes': vol2['id']}
        self.groups_client.update_group(grp['id'], **grp_params)

        # Wait for group status to become available
        waiters.wait_for_volume_resource_status(
            self.groups_client, grp['id'], 'available')

        # Get volumes in the group
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        grp_vols = [v for v in vols if v['group_id'] == grp['id']]
        self.assertEqual(2, len(grp_vols))


class GroupsV314Test(base.BaseVolumeAdminTest):
    """Tests of volume groups with microversion greater than 3.13"""

    volume_min_microversion = '3.14'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('2424af8c-7851-4888-986a-794b10c3210e')
    def test_create_group_from_group(self):
        """Test creating volume group from volume group"""
        # Create volume type
        volume_type = self.create_volume_type()

        # Create group type
        group_type = self.create_group_type()

        # Create Group
        grp = self.create_group(group_type=group_type['id'],
                                volume_types=[volume_type['id']])

        prefix = CONF.resource_name_prefix
        # Create volume is instance level, can not be deleted before group.
        # Volume delete handled by delete_group method, cleanup method.
        params = {'name': data_utils.rand_name(prefix=prefix, name="volume"),
                  'volume_type': volume_type['id'],
                  'group_id': grp['id'],
                  'size': CONF.volume.volume_size}
        vol = self.volumes_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(
            self.volumes_client, vol['id'], 'available')

        # Create Group from Group
        grp_name2 = data_utils.rand_name(prefix=prefix, name='Group_from_grp')
        grp2 = self.groups_client.create_group_from_source(
            source_group_id=grp['id'], name=grp_name2)['group']
        self.addCleanup(self.delete_group, grp2['id'])
        self.assertEqual(grp_name2, grp2['name'])
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        for vol in vols:
            if vol['group_id'] == grp2['id']:
                waiters.wait_for_volume_resource_status(
                    self.volumes_client, vol['id'], 'available')
        waiters.wait_for_volume_resource_status(
            self.groups_client, grp2['id'], 'available')


class GroupsV320Test(base.BaseVolumeAdminTest):
    """Tests of volume groups with microversion greater than 3.19"""

    volume_min_microversion = '3.20'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('b20c696b-0cbc-49a5-8b3a-b1fb9338f45c')
    def test_reset_group_status(self):
        """Test resetting volume group status to creating/available/error"""
        # Create volume type
        volume_type = self.create_volume_type()

        # Create group type
        group_type = self.create_group_type()

        # Create group
        group = self.create_group(group_type=group_type['id'],
                                  volume_types=[volume_type['id']])

        # Reset group status
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.groups_client, group['id'], 'available')
        self.addCleanup(self.admin_groups_client.reset_group_status,
                        group['id'], 'available')
        for status in ['creating', 'available', 'error']:
            self.admin_groups_client.reset_group_status(group['id'], status)
            waiters.wait_for_volume_resource_status(
                self.groups_client, group['id'], status)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import keypairs

get_keypair = copy.deepcopy(keypairs.get_keypair)
get_keypair['response_body']['properties']['keypair'][
    'properties'].update({'type': {'type': 'string'}})
get_keypair['response_body']['properties']['keypair'][
    'required'].append('type')

create_keypair = copy.deepcopy(keypairs.create_keypair)
create_keypair['status_code'] = [201]
create_keypair['response_body']['properties']['keypair'][
    'properties'].update({'type': {'type': 'string'}})
create_keypair['response_body']['properties']['keypair'][
    'required'].append('type')

delete_keypair = {
    'status_code': [204],
}

list_keypairs = copy.deepcopy(keypairs.list_keypairs)
list_keypairs['response_body']['properties']['keypairs'][
    'items']['properties']['keypair'][
    'properties'].update({'type': {'type': 'string'}})
list_keypairs['response_body']['properties']['keypairs'][
    'items']['properties']['keypair']['required'].append('type')



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import services_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestServicesClient(base.BaseServiceTest):
    FAKE_CREATE_SERVICE = {
        "service": {
            "type": "compute",
            "name": "compute2",
            "description": "Compute service 2"
            }
        }

    FAKE_SERVICE_INFO = {
        "service": {
            "description": "Keystone Identity Service",
            "enabled": True,
            "id": "686766",
            "links": {
                "self": "http://example.com/identity/v3/services/686766"
                },
            "name": "keystone",
            "type": "identity"
            }
        }

    FAKE_LIST_SERVICES = {
        "links": {
            "next": None,
            "previous": None,
            "self": "http://example.com/identity/v3/services"
            },
        "services": [
            {
                "description": "Nova Compute Service",
                "enabled": True,
                "id": "1999c3",
                "links": {
                    "self": "http://example.com/identity/v3/services/1999c3"
                    },
                "name": "nova",
                "type": "compute"
                },
            {
                "description": "Cinder Volume Service V2",
                "enabled": True,
                "id": "392166",
                "links": {
                    "self": "http://example.com/identity/v3/services/392166"
                    },
                "name": "cinderv2",
                "type": "volumev2"
                },
            {
                "description": "Neutron Service",
                "enabled": True,
                "id": "4fe41a",
                "links": {
                    "self": "http://example.com/identity/v3/services/4fe41a"
                    },
                "name": "neutron",
                "type": "network"
                }
        ]
    }

    def setUp(self):
        super(TestServicesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = services_client.ServicesClient(fake_auth, 'identity',
                                                     'regionOne')

    def _test_create_service(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_service,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_SERVICE,
            bytes_body,
            status=201)

    def _test_show_service(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_service,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVICE_INFO,
            bytes_body,
            service_id="686766")

    def _test_list_services(self, bytes_body=False, mock_args='services',
                            **params):
        self.check_service_client_function(
            self.client.list_services,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_SERVICES,
            bytes_body,
            mock_args=[mock_args],
            **params)

    def _test_update_service(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_service,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_SERVICE_INFO,
            bytes_body,
            service_id="686766")

    def test_create_service_with_str_body(self):
        self._test_create_service()

    def test_create_service_with_bytes_body(self):
        self._test_create_service(bytes_body=True)

    def test_show_service_with_str_body(self):
        self._test_show_service()

    def test_show_service_with_bytes_body(self):
        self._test_show_service(bytes_body=True)

    def test_list_services_with_str_body(self):
        self._test_list_services()

    def test_list_services_with_bytes_body(self):
        self._test_list_services(bytes_body=True)

    def test_list_services_with_params(self):
        self._test_list_services(
            type='fake-type', mock_args='services?type=fake-type')

    def test_update_service_with_str_body(self):
        self._test_update_service()

    def test_update_service_with_bytes_body(self):
        self._test_update_service(bytes_body=True)

    def test_delete_service(self):
        self.check_service_client_function(
            self.client.delete_service,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            service_id="686766",
            status=204)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.api.identity import base
from tempest import clients
from tempest import config
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class TestDefaultProjectId(base.BaseIdentityV3AdminTest):
    """Test creating a token without project will default to user's project"""

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources()
        super(TestDefaultProjectId, cls).setup_credentials()

    def _delete_domain(self, domain_id):
        # It is necessary to disable the domain before deleting,
        # or else it would result in unauthorized error
        self.domains_client.update_domain(domain_id, enabled=False)
        self.domains_client.delete_domain(domain_id)

    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an immutable user '
                      'source and solely provides read-only access to users.')
    @decorators.idempotent_id('d6110661-6a71-49a7-a453-b5e26640ff6d')
    def test_default_project_id(self):
        """Creating a token without project will default to user's project"""
        # create a domain
        dom_name = data_utils.rand_name(
            name='dom', prefix=CONF.resource_name_prefix)
        domain_body = self.domains_client.create_domain(
            name=dom_name)['domain']
        dom_id = domain_body['id']
        self.addCleanup(self._delete_domain, dom_id)

        # create a project in the domain
        proj_body = self.setup_test_project(domain_id=dom_id)
        proj_id = proj_body['id']
        self.assertEqual(proj_body['domain_id'], dom_id,
                         "project " + proj_body['name'] +
                         "doesn't have domain id " + dom_id)

        # create a user in the domain, with the previous project as his
        # default project
        user_name = data_utils.rand_name(
            name='user', prefix=CONF.resource_name_prefix)
        user_pass = data_utils.rand_password()
        user_body = self.users_client.create_user(
            name=user_name,
            password=user_pass,
            domain_id=dom_id,
            default_project_id=proj_id)['user']
        user_id = user_body['id']
        self.addCleanup(self.users_client.delete_user, user_id)
        self.assertEqual(user_body['domain_id'], dom_id,
                         "user " + user_name +
                         "doesn't have domain id " + dom_id)

        # get roles and find the admin role
        admin_role = self.get_role_by_name(CONF.identity.admin_role)
        admin_role_id = admin_role['id']

        # grant the admin role to the user on his project
        self.roles_client.create_user_role_on_project(proj_id, user_id,
                                                      admin_role_id)

        # create a new client with user's credentials (NOTE: unscoped token!)
        creds = auth.KeystoneV3Credentials(username=user_name,
                                           password=user_pass,
                                           user_domain_name=dom_name)
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        admin_client = clients.Manager(credentials=creds)

        # verify the user's token and see that it is scoped to the project
        token, _ = admin_client.auth_provider.get_auth()
        result = admin_client.identity_v3_client.show_token(token)['token']
        self.assertEqual(result['project']['domain']['id'], dom_id)
        self.assertEqual(result['project']['id'], proj_id)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.api.compute import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


def get_subnets(count=2):
    """Returns a list of requested subnets from project_network_cidr block.

    Args:
        count (int):    Number of blocks required.

    Returns:
        CIDRs as a list of strings
            e.g. ['19.80.0.0/24', '19.86.0.0/24']
    """
    default_rtn = ['19.80.0.0/24', '19.86.0.0/24']
    _net = netaddr.IPNetwork(CONF.network.project_network_cidr)

    # Split the subnet into the requested number of smaller subnets.
    sub_prefix_len = (32 - _net.prefixlen) // count
    if sub_prefix_len < 1:
        return default_rtn

    _new_cidr = _net.prefixlen + sub_prefix_len
    return [str(net) for _, net in zip(range(count), _net.subnet(_new_cidr))]


class ServersTestMultiNic(base.BaseV2ComputeTest):
    """Test multiple networks in servers"""

    @classmethod
    def skip_checks(cls):
        super(ServersTestMultiNic, cls).skip_checks()
        if not CONF.service_available.neutron:
            raise cls.skipException('Neutron service must be available.')

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(ServersTestMultiNic, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ServersTestMultiNic, cls).setup_clients()
        cls.client = cls.servers_client
        cls.networks_client = cls.os_primary.networks_client
        cls.subnets_client = cls.os_primary.subnets_client

    def _create_net_subnet_ret_net_from_cidr(self, cidr):
        name_net = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__)
        net = self.networks_client.create_network(name=name_net)
        self.addCleanup(self.networks_client.delete_network,
                        net['network']['id'])

        subnet = self.subnets_client.create_subnet(
            network_id=net['network']['id'],
            cidr=cidr,
            ip_version=4)
        self.addCleanup(self.subnets_client.delete_subnet,
                        subnet['subnet']['id'])
        return net

    @decorators.idempotent_id('0578d144-ed74-43f8-8e57-ab10dbf9b3c2')
    def test_verify_multiple_nics_order(self):
        """Test verifying multiple networks order in server

        The networks order given at the server creation is preserved within
        the server.
        """
        _cidrs = get_subnets()
        net1 = self._create_net_subnet_ret_net_from_cidr(_cidrs[0])
        net2 = self._create_net_subnet_ret_net_from_cidr(_cidrs[1])

        networks = [{'uuid': net1['network']['id']},
                    {'uuid': net2['network']['id']}]

        server_multi_nics = self.create_test_server(
            networks=networks, wait_until='ACTIVE')

        # Cleanup server; this is needed in the test case because with the LIFO
        # nature of the cleanups, if we don't delete the server first, the port
        # will still be part of the subnet and we'll get a 409 from Neutron
        # when trying to delete the subnet. The tear down in the base class
        # will try to delete the server and get a 404 but it's ignored so
        # we're OK.
        self.addCleanup(self.delete_server, server_multi_nics['id'])

        addresses = (self.client.list_addresses(server_multi_nics['id'])
                     ['addresses'])

        # We can't predict the ip addresses assigned to the server on networks.
        # So we check if the first address is in first network, similarly
        # second address is in second network.
        addr = [addresses[net1['network']['name']][0]['addr'],
                addresses[net2['network']['name']][0]['addr']]
        networks = [netaddr.IPNetwork(_cidrs[0]),
                    netaddr.IPNetwork(_cidrs[1])]
        for address, network in zip(addr, networks):
            self.assertIn(address, network)

    @decorators.idempotent_id('1678d144-ed74-43f8-8e57-ab10dbf9b3c2')
    def test_verify_duplicate_network_nics(self):
        """Test multiple duplicate networks can be used to create server

        Creating server with networks [net1, net2, net1], the server can
        be created successfully and all three networks are in the server
        addresses.
        """
        # Verify that server creation does not fail when more than one nic
        # is created on the same network.
        _cidrs = get_subnets()
        net1 = self._create_net_subnet_ret_net_from_cidr(_cidrs[0])
        net2 = self._create_net_subnet_ret_net_from_cidr(_cidrs[1])

        networks = [{'uuid': net1['network']['id']},
                    {'uuid': net2['network']['id']},
                    {'uuid': net1['network']['id']}]

        server_multi_nics = self.create_test_server(
            networks=networks, wait_until='ACTIVE')
        self.addCleanup(self.delete_server, server_multi_nics['id'])

        addresses = (self.client.list_addresses(server_multi_nics['id'])
                     ['addresses'])

        addr = [addresses[net1['network']['name']][0]['addr'],
                addresses[net2['network']['name']][0]['addr'],
                addresses[net1['network']['name']][1]['addr']]
        networks = [netaddr.IPNetwork(_cidrs[0]),
                    netaddr.IPNetwork(_cidrs[1]),
                    netaddr.IPNetwork(_cidrs[0])]
        for address, network in zip(addr, networks):
            self.assertIn(address, network)



# Copyright 2016 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy


def get_image_meta_from_headers(resp):
    meta = {'properties': {}}
    for key in resp.response:
        value = resp.response[key]
        if key.startswith('x-image-meta-property-'):
            _key = key[22:]
            meta['properties'][_key] = value
        elif key.startswith('x-image-meta-'):
            _key = key[13:]
            meta[_key] = value

    for key in ['is_public', 'protected', 'deleted']:
        if key in meta:
            meta[key] = meta[key].strip().lower() in ('t', 'true', 'yes', '1')

    for key in ['size', 'min_ram', 'min_disk']:
        if key in meta:
            try:
                meta[key] = int(meta[key])
            except ValueError:
                pass
    return meta


def image_meta_to_headers(**metadata):
    headers = {}
    fields_copy = copy.deepcopy(metadata)

    copy_from = fields_copy.pop('copy_from', None)
    purge = fields_copy.pop('purge_props', None)

    if purge is not None:
        headers['x-glance-registry-purge-props'] = purge

    if copy_from is not None:
        headers['x-glance-api-copy-from'] = copy_from

    for key, value in fields_copy.pop('properties', {}).items():
        headers['x-image-meta-property-%s' % key] = str(value)

    for key, value in fields_copy.pop('api', {}).items():
        headers['x-glance-api-property-%s' % key] = str(value)

    for key, value in fields_copy.items():
        headers['x-image-meta-%s' % key] = str(value)

    return headers



# Copyright 2015 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class MetadataNamespacesTest(base.BaseV2ImageAdminTest):
    """Test the Metadata definition Namespaces basic functionality"""

    @decorators.idempotent_id('319b765e-7f3d-4b3d-8b37-3ca3876ee768')
    def test_basic_metadata_definition_namespaces(self):
        """Test operations of image metadata definition namespaces"""
        # get the available resource types and use one resource_type
        body = self.resource_types_client.list_resource_types()
        resource_name = body['resource_types'][0]['name']
        name = [{'name': resource_name}]
        namespace_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='namespace')
        # create the metadef namespace
        body = self.namespaces_client.create_namespace(
            namespace=namespace_name,
            visibility='public',
            description='Tempest',
            display_name=namespace_name,
            resource_type_associations=name,
            protected=True)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self._cleanup_namespace, namespace_name)
        # list namespaces
        bodys = self.namespaces_client.list_namespaces()['namespaces']
        body = [namespace['namespace'] for namespace in bodys]
        self.assertIn(namespace_name, body)
        # get namespace details
        body = self.namespaces_client.show_namespace(namespace_name)
        self.assertEqual(namespace_name, body['namespace'])
        self.assertEqual('public', body['visibility'])
        # unable to delete protected namespace
        self.assertRaises(lib_exc.Forbidden,
                          self.namespaces_client.delete_namespace,
                          namespace_name)
        # update the visibility to private and protected to False
        body = self.namespaces_client.update_namespace(
            namespace=namespace_name,
            description='Tempest',
            visibility='private',
            display_name=namespace_name,
            protected=False)
        self.assertEqual('private', body['visibility'])
        self.assertEqual(False, body['protected'])
        # now able to delete the non-protected namespace
        self.namespaces_client.delete_namespace(namespace_name)
        self.namespaces_client.wait_for_resource_deletion(namespace_name)

    def _cleanup_namespace(self, namespace_name):
        body = self.namespaces_client.show_namespace(namespace_name)
        self.assertEqual(namespace_name, body['namespace'])
        body = self.namespaces_client.update_namespace(
            namespace=namespace_name,
            description='Tempest',
            visibility='private',
            display_name=namespace_name,
            protected=False)
        self.namespaces_client.delete_namespace(namespace_name)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute.floating_ips import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class FloatingIPDetailsNegativeTestJSON(base.BaseFloatingIPsTest):
    """Negative tests of floating ip detail

    Negative tests of floating ip detail with compute microversion less
    than 2.36.
    """

    max_microversion = '2.35'

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7ab18834-4a4b-4f28-a2c5-440579866695')
    def test_get_nonexistent_floating_ip_details(self):
        """Test getting non existent floating ip should fail"""
        # Creating a non-existent floatingIP id
        if CONF.service_available.neutron:
            non_exist_id = data_utils.rand_uuid()
        else:
            non_exist_id = data_utils.rand_int_id(start=999)
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_floating_ip, non_exist_id)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

create_get_volume = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volume': {
                'type': 'object',
                'properties': {
                    'id': {'type': 'string'},
                    'status': {'type': 'string'},
                    'displayName': {'type': ['string', 'null']},
                    'availabilityZone': {'type': 'string'},
                    'createdAt': parameter_types.date_time,
                    'displayDescription': {'type': ['string', 'null']},
                    'volumeType': {'type': ['string', 'null']},
                    'snapshotId': {'type': ['string', 'null']},
                    'metadata': {'type': 'object'},
                    'size': {'type': 'integer'},
                    'attachments': {
                        'type': 'array',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'id': {'type': 'string'},
                                'device': {'type': 'string'},
                                'volumeId': {'type': 'string'},
                                'serverId': {'type': 'string'}
                            },
                            'additionalProperties': False,
                            # NOTE- If volume is not attached to any server
                            # then, 'attachments' attributes comes as array
                            # with empty objects "[{}]" due to that elements
                            # of 'attachments' cannot defined as 'required'.
                            # If it would come as empty array "[]" then,
                            # those elements can be defined as 'required'.
                        }
                    },
                    'os-vol-host-attr:host': {'type': 'string'},
                },
                'additionalProperties': False,
                'required': ['id', 'status', 'displayName', 'availabilityZone',
                             'createdAt', 'displayDescription', 'volumeType',
                             'snapshotId', 'metadata', 'size', 'attachments']
            }
        },
        'additionalProperties': False,
        'required': ['volume']
    }
}

list_volumes = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volumes': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': 'string'},
                        'status': {'type': 'string'},
                        'displayName': {'type': ['string', 'null']},
                        'availabilityZone': {'type': 'string'},
                        'createdAt': parameter_types.date_time,
                        'displayDescription': {'type': ['string', 'null']},
                        'volumeType': {'type': ['string', 'null']},
                        'snapshotId': {'type': ['string', 'null']},
                        'metadata': {'type': 'object'},
                        'size': {'type': 'integer'},
                        'attachments': {
                            'type': 'array',
                            'items': {
                                'type': 'object',
                                'properties': {
                                    'id': {'type': 'string'},
                                    'device': {'type': 'string'},
                                    'volumeId': {'type': 'string'},
                                    'serverId': {'type': 'string'}
                                },
                                'additionalProperties': False,
                                # NOTE- If volume is not attached to any server
                                # then, 'attachments' attributes comes as array
                                # with empty object "[{}]" due to that elements
                                # of 'attachments' cannot defined as 'required'
                                # If it would come as empty array "[]" then,
                                # those elements can be defined as 'required'.
                            }
                        }
                    },
                    'additionalProperties': False,
                    'required': ['id', 'status', 'displayName',
                                 'availabilityZone', 'createdAt',
                                 'displayDescription', 'volumeType',
                                 'snapshotId', 'metadata', 'size',
                                 'attachments']
                }
            }
        },
        'additionalProperties': False,
        'required': ['volumes']
    }
}

delete_volume = {
    'status_code': [202]
}



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import custom_matchers
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF


class TestMinimumBasicScenario(manager.ScenarioTest):

    """This is a basic minimum scenario test.

    These tests below:
    * across the multiple components
    * as a regular user
    * with and without optional parameters
    * check command outputs
    """

    @classmethod
    def skip_checks(cls):
        super(TestMinimumBasicScenario, cls).skip_checks()
        if not CONF.service_available.cinder:
            raise cls.skipException("Cinder is not available")

    def nova_show(self, server):
        got_server = (self.servers_client.show_server(server['id'])
                      ['server'])
        excluded_keys = ['OS-EXT-AZ:availability_zone']
        # Exclude these keys because of LP:#1486475
        excluded_keys.extend(['OS-EXT-STS:power_state', 'updated'])
        self.assertThat(
            server, custom_matchers.MatchesDictExceptForKeys(
                got_server, excluded_keys=excluded_keys))

    def cinder_show(self, volume):
        got_volume = self.volumes_client.show_volume(volume['id'])['volume']
        # Exclude updated_at because of bug 1838202.
        excluded_keys = ['updated_at']
        self.assertThat(
            volume, custom_matchers.MatchesDictExceptForKeys(
                got_volume, excluded_keys=excluded_keys))

    def nova_reboot(self, server, hard=False):
        self.servers_client.reboot_server(server['id'],
                                          type="HARD" if hard else "SOFT")
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')

    def check_disks(self):
        # NOTE(andreaf) The device name may be different on different guest OS
        disks = self.linux_client.get_disks()
        self.assertEqual(1, disks.count(CONF.compute.volume_device_name))

    @decorators.attr(type='slow')
    @decorators.idempotent_id('bdbb5441-9204-419d-a225-b4fdbfb1a1a8')
    @utils.services('compute', 'volume', 'image', 'network')
    def test_minimum_basic_scenario(self):
        """This is a basic minimum scenario with multiple components

        Steps:
        1. Create image
        2. Create keypair
        3. Boot instance with keypair and get list of instances
        4. Create volume and show list of volumes
        5. Attach volume to instance and getlist of volumes
        6. Add IP to instance
        7. Create and add security group to instance
        8. Check SSH connection to instance
        9. Reboot instance
        10. Check SSH connection to instance after reboot
        """
        image = self.image_create()
        keypair = self.create_keypair()

        server = self.create_server(image_id=image, key_name=keypair['name'])
        servers = self.servers_client.list_servers()['servers']
        self.assertIn(server['id'], [x['id'] for x in servers])

        self.nova_show(server)

        volume = self.create_volume()
        volumes = self.volumes_client.list_volumes()['volumes']
        self.assertIn(volume['id'], [x['id'] for x in volumes])

        self.cinder_show(volume)

        volume = self.nova_volume_attach(server, volume)
        self.addCleanup(self.nova_volume_detach, server, volume)
        self.cinder_show(volume)

        floating_ip = None
        server = self.servers_client.show_server(server['id'])['server']
        if (CONF.network_feature_enabled.floating_ips and
                CONF.network.floating_network_name):
            fip = self.create_floating_ip(server)
            floating_ip = self.associate_floating_ip(
                fip, server)
            waiters.wait_for_server_floating_ip(self.servers_client,
                                                server, floating_ip)
            ssh_ip = floating_ip['floating_ip_address']
        else:
            ssh_ip = self.get_server_ip(server)

        self.create_and_add_security_group_to_server(server)

        # check that we can SSH to the server before reboot
        self.linux_client = self.get_remote_client(
            ssh_ip, private_key=keypair['private_key'],
            server=server)

        self.nova_reboot(server)

        # check that we can SSH to the server after reboot
        # (both connections are part of the scenario)
        self.linux_client = self.get_remote_client(
            ssh_ip, private_key=keypair['private_key'],
            server=server)

        self.check_disks()

        if floating_ip:
            # delete the floating IP, this should refresh the server addresses
            self.disassociate_floating_ip(floating_ip)
            waiters.wait_for_server_floating_ip(
                self.servers_client, server, floating_ip,
                wait_for_disassociate=True)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('a8fd48ec-1d01-4895-b932-02321661ec1e')
    @testtools.skipUnless(CONF.volume_feature_enabled.snapshot,
                          "Cinder volume snapshots are disabled")
    @utils.services('compute', 'volume', 'image', 'network')
    def test_minimum_basic_instance_hard_reboot_after_vol_snap_deletion(self):
        """Test compute hard reboot after volume snapshot deleted

        Steps:
        1. Create image
        2. Create keypair
        3. Boot instance with keypair and get list of instances
        4. Create volume and show list of volumes
        5. Attach volume to instance and getlist of volumes
        6. Create a snapshot from volume
        7. Add IP to instance
        8. Create and add security group to instance
        9. Check SSH connection to instance
        10. Write data timestamp to the attached volume
        11. Delete volume snapshot before reboot instance
        12. Reboot instance (HARD)
        13. Check SSH connection to instance after reboot
        14. Verify attached disk data timestamp post instance reboot
        """
        image = self.image_create()
        keypair = self.create_keypair()

        server = self.create_server(image_id=image, key_name=keypair['name'])
        servers = self.servers_client.list_servers()['servers']
        self.assertIn(server['id'], [x['id'] for x in servers])

        self.nova_show(server)

        volume = self.create_volume()
        volumes = self.volumes_client.list_volumes()['volumes']
        self.assertIn(volume['id'], [x['id'] for x in volumes])

        self.cinder_show(volume)

        volume = self.nova_volume_attach(server, volume)
        self.addCleanup(self.nova_volume_detach, server, volume)
        snapshot = self.create_volume_snapshot(volume['id'], force=True)
        self.cinder_show(volume)

        floating_ip = None
        server = self.servers_client.show_server(server['id'])['server']
        if (CONF.network_feature_enabled.floating_ips and
                CONF.network.floating_network_name):
            fip = self.create_floating_ip(server)
            floating_ip = self.associate_floating_ip(
                fip, server)
            waiters.wait_for_server_floating_ip(self.servers_client, server,
                                                floating_ip)
            ssh_ip = floating_ip['floating_ip_address']
        else:
            ssh_ip = self.get_server_ip(server)

        self.create_and_add_security_group_to_server(server)

        # check that we can SSH to the server before reboot
        self.linux_client = self.get_remote_client(
            ssh_ip, private_key=keypair['private_key'],
            server=server)

        # write data to the volume before reboot instance
        timestamp_before = self.create_timestamp(
            ssh_ip, private_key=keypair['private_key'], server=server)
        # delete the snapshot before rebooting the instance
        self.snapshots_client.delete_snapshot(snapshot['id'])
        self.snapshots_client.wait_for_resource_deletion(snapshot['id'])
        self.nova_reboot(server, hard=True)

        # check that we can SSH to the server after reboot
        # (both connections are part of the scenario)
        self.linux_client = self.get_remote_client(
            ssh_ip, private_key=keypair['private_key'],
            server=server)

        self.check_disks()
        timestamp_after = self.get_timestamp(
            ssh_ip, private_key=keypair['private_key'], server=server)
        self.assertEqual(timestamp_before, timestamp_after)
        if floating_ip:
            # delete the floating IP, this should refresh the server addresses
            self.disassociate_floating_ip(floating_ip)
            waiters.wait_for_server_floating_ip(
                self.servers_client, server, floating_ip,
                wait_for_disassociate=True)



# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

import fixtures
from oslo_config import cfg

from tempest.common.utils.linux import remote_client
from tempest import config
from tempest.lib import exceptions as lib_exc
from tempest.tests import base
from tempest.tests import fake_config


SERVER = {
    'id': 'server_uuid',
    'name': 'fake_server',
    'status': 'ACTIVE'
}

BROKEN_SERVER = {
    'id': 'broken_server_uuid',
    'name': 'broken_server',
    'status': 'ERROR'
}


class FakeServersClient(object):

    CONSOLE_OUTPUT = "Console output for %s"

    def get_console_output(self, server_id):
        status = 'ERROR'
        for s in SERVER, BROKEN_SERVER:
            if s['id'] == server_id:
                status = s['status']
        if status == 'ERROR':
            raise lib_exc.BadRequest('Server in ERROR state')
        else:
            return dict(output=self.CONSOLE_OUTPUT % server_id)


class TestRemoteClient(base.TestCase):
    def setUp(self):
        super(TestRemoteClient, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)
        cfg.CONF.set_default('ip_version_for_ssh', 4, group='validation')
        cfg.CONF.set_default('network_for_ssh', 'public', group='validation')
        cfg.CONF.set_default('connect_timeout', 1, group='validation')

        self.conn = remote_client.RemoteClient('127.0.0.1', 'user', 'pass')
        self.ssh_mock = self.useFixture(fixtures.MockPatchObject(self.conn,
                                                                 'ssh_client'))

    def test_write_to_console_regular_str(self):
        self.conn.write_to_console('test')
        self._assert_exec_called_with(
            'sudo sh -c "echo \\"test\\" >/dev/console"')

    def _test_write_to_console_helper(self, message, expected_call):
        self.conn.write_to_console(message)
        self._assert_exec_called_with(expected_call)

    def test_write_to_console_special_chars(self):
        self._test_write_to_console_helper(
            r'\`',
            'sudo sh -c "echo \\"\\\\\\`\\" >/dev/console"')
        self.conn.write_to_console('$')
        self._assert_exec_called_with(
            'sudo sh -c "echo \\"\\\\$\\" >/dev/console"')

    # NOTE(maurosr): The tests below end up closer to an output format
    # assurance than a test since it's basically using comand_exec to format
    # the information using gnu/linux tools.

    def _assert_exec_called_with(self, cmd):
        cmd = "set -eu -o pipefail; PATH=$PATH:/sbin:/usr/sbin; " + cmd
        self.ssh_mock.mock.exec_command.assert_called_with(cmd)

    def test_get_disks(self):
        output_lsblk = """\
NAME       MAJ:MIN    RM          SIZE RO TYPE MOUNTPOINT
sda          8:0       0  128035676160  0 disk
sdb          8:16      0 1000204886016  0 disk
sr0         11:0       1    1073741312  0 rom"""
        result = """\
NAME       MAJ:MIN    RM          SIZE RO TYPE MOUNTPOINT
sda          8:0       0  128035676160  0 disk
sdb          8:16      0 1000204886016  0 disk"""

        self.ssh_mock.mock.exec_command.return_value = output_lsblk
        self.assertEqual(self.conn.get_disks(), result)
        self._assert_exec_called_with('lsblk -lb --nodeps')

    def test_list_disks(self):
        output_lsblk = """\
NAME       MAJ:MIN    RM          SIZE RO TYPE MOUNTPOINT
sda          8:0       0  128035676160  0 disk
sdb          8:16      0 1000204886016  0 disk
sr0         11:0       1    1073741312  0 rom"""
        disk_list = ['sda', 'sdb']
        self.ssh_mock.mock.exec_command.return_value = output_lsblk
        self.assertEqual(self.conn.list_disks(), disk_list)

    def test_get_boot_time(self):
        booted_at = 10000
        uptime_sec = 5000.02
        self.ssh_mock.mock.exec_command.return_value = uptime_sec
        self.useFixture(fixtures.MockPatchObject(
            time, 'time', return_value=booted_at + uptime_sec))
        self.assertEqual(self.conn.get_boot_time(),
                         time.localtime(booted_at))
        self._assert_exec_called_with('cut -f1 -d. /proc/uptime')

    def test_ping_host(self):
        ping_response = """PING localhost (127.0.0.1) 70(98) bytes of data.
78 bytes from localhost (127.0.0.1): icmp_req=1 ttl=64 time=0.048 ms
78 bytes from localhost (127.0.0.1): icmp_req=2 ttl=64 time=0.048 ms

--- localhost ping statistics ---
2 packets transmitted, 2 received, 0% packet loss, time 0ms
rtt min/avg/max/mdev = 0.048/0.048/0.048/0.000 ms"""
        self.ssh_mock.mock.exec_command.return_value = ping_response
        self.assertEqual(self.conn.ping_host('127.0.0.1', count=2, size=70),
                         ping_response)
        self._assert_exec_called_with('ping -c2 -w2 -s70 127.0.0.1')

    def test_get_mac_address(self):
        macs = """0a:0b:0c:0d:0e:0f
a0:b0:c0:d0:e0:f0"""
        self.ssh_mock.mock.exec_command.return_value = macs

        self.assertEqual(self.conn.get_mac_address(), macs)
        self._assert_exec_called_with(
            "ip addr | awk '/ether/ {print $2}'")


class TestRemoteClientWithServer(base.TestCase):

    server = SERVER

    def setUp(self):
        super(TestRemoteClientWithServer, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)
        cfg.CONF.set_default('ip_version_for_ssh', 4, group='validation')
        cfg.CONF.set_default('network_for_ssh', 'public',
                             group='validation')
        cfg.CONF.set_default('connect_timeout', 1, group='validation')
        cfg.CONF.set_default('console_output', True,
                             group='compute-feature-enabled')

        self.conn = remote_client.RemoteClient(
            '127.0.0.1', 'user', 'pass',
            server=self.server, servers_client=FakeServersClient())
        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.ssh.Client._get_ssh_connection',
            side_effect=lib_exc.SSHTimeout(host='127.0.0.1',
                                           user='user',
                                           password='pass')))
        self.log = self.useFixture(fixtures.FakeLogger(
            name='tempest.lib.common.utils.linux.remote_client',
            level='DEBUG'))

    def test_validate_debug_ssh_console(self):
        self.assertRaises(lib_exc.SSHTimeout,
                          self.conn.validate_authentication)
        msg = 'Caller: %s. Timeout trying to ssh to server %s' % (
            'TestRemoteClientWithServer:test_validate_debug_ssh_console',
            self.server)
        self.assertIn(msg, self.log.output)
        self.assertIn('Console output for', self.log.output)

    def test_exec_command_debug_ssh_console(self):
        self.assertRaises(lib_exc.SSHTimeout,
                          self.conn.exec_command, 'fake command')
        self.assertIn('fake command', self.log.output)
        msg = 'Caller: %s. Timeout trying to ssh to server %s' % (
            'TestRemoteClientWithServer:test_exec_command_debug_ssh_console',
            self.server)
        self.assertIn(msg, self.log.output)
        self.assertIn('Console output for', self.log.output)


class TestRemoteClientWithBrokenServer(TestRemoteClientWithServer):

    server = BROKEN_SERVER

    def test_validate_debug_ssh_console(self):
        self.assertRaises(lib_exc.SSHTimeout,
                          self.conn.validate_authentication)
        msg = 'Caller: %s. Timeout trying to ssh to server %s' % (
            'TestRemoteClientWithBrokenServer:test_validate_debug_ssh_console',
            self.server)
        self.assertIn(msg, self.log.output)
        msg = 'Could not get console_log for server %s' % self.server['id']
        self.assertIn(msg, self.log.output)

    def test_exec_command_debug_ssh_console(self):
        self.assertRaises(lib_exc.SSHTimeout,
                          self.conn.exec_command, 'fake command')
        self.assertIn('fake command', self.log.output)
        caller = ":".join(['TestRemoteClientWithBrokenServer',
                           'test_exec_command_debug_ssh_console'])
        msg = 'Caller: %s. Timeout trying to ssh to server %s' % (
            caller, self.server)
        self.assertIn(msg, self.log.output)
        msg = 'Could not get console_log for server %s' % self.server['id']
        self.assertIn(msg, self.log.output)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

interface_common_info = {
    'type': 'object',
    'properties': {
        'port_state': {'type': 'string'},
        'fixed_ips': {
            'type': 'array',
            'items': {
                'type': 'object',
                'properties': {
                    'subnet_id': {
                        'type': 'string',
                        'format': 'uuid'
                    },
                    'ip_address': parameter_types.ip_address
                },
                'additionalProperties': False,
                'required': ['subnet_id', 'ip_address']
            }
        },
        'port_id': {'type': 'string', 'format': 'uuid'},
        'net_id': {'type': 'string', 'format': 'uuid'},
        'mac_addr': parameter_types.mac_address
    },
    'additionalProperties': False,
    'required': ['port_state', 'fixed_ips', 'port_id', 'net_id', 'mac_addr']
}

get_create_interfaces = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'interfaceAttachment': interface_common_info
        },
        'additionalProperties': False,
        'required': ['interfaceAttachment']
    }
}

list_interfaces = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'interfaceAttachments': {
                'type': 'array',
                'items': interface_common_info
            }
        },
        'additionalProperties': False,
        'required': ['interfaceAttachments']
    }
}

delete_interface = {
    'status_code': [202]
}



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import transfers as schema
from tempest.lib.api_schema.response.volume.v3_55 \
    import transfers as schemav355
from tempest.lib.api_schema.response.volume.v3_57 \
    import transfers as schemav357
from tempest.lib.common import rest_client
from tempest.lib.services.volume import base_client


class TransfersClient(base_client.BaseClient):
    """Client class to send CRUD Volume Transfer API requests"""

    schema_versions_info = [
        {'min': None, 'max': '3.54', 'schema': schema},
        {'min': '3.55', 'max': '3.56', 'schema': schemav355},
        {'min': '3.57', 'max': None, 'schema': schemav357}
    ]

    resource_path = 'os-volume-transfer'

    def create_volume_transfer(self, **kwargs):
        """Create a volume transfer.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#create-a-volume-transfer
        """
        post_body = json.dumps({'transfer': kwargs})
        resp, body = self.post(self.resource_path, post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_volume_transfer, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_volume_transfer(self, transfer_id):
        """Returns the details of a volume transfer."""
        url = "%s/%s" % (self.resource_path, transfer_id)
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.show_volume_transfer, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_volume_transfers(self, detail=False, **params):
        """List all the volume transfers created.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#list-volume-transfers-for-a-project
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#list-volume-transfers-and-details
        """
        url = self.resource_path
        schema = self.get_schema(self.schema_versions_info)
        schema_list_transfers = schema.list_volume_transfers_no_detail
        if detail:
            url += '/detail'
            schema_list_transfers = schema.list_volume_transfers_with_detail
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema_list_transfers, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_volume_transfer(self, transfer_id):
        """Delete a volume transfer."""
        resp, body = self.delete("%s/%s" % (self.resource_path, transfer_id))
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.delete_volume_transfer, resp, body)
        return rest_client.ResponseBody(resp, body)

    def accept_volume_transfer(self, transfer_id, **kwargs):
        """Accept a volume transfer.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#accept-a-volume-transfer
        """
        url = '%s/%s/accept' % (self.resource_path, transfer_id)
        post_body = json.dumps({'accept': kwargs})
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.accept_volume_transfer, resp, body)
        return rest_client.ResponseBody(resp, body)


class TransfersV355Client(TransfersClient):
    """Client class to send CRUD for the "new" Transfers API (mv 3.55)"""
    resource_path = 'volume-transfers'



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from oslo_concurrency import lockutils
from oslo_log import log as logging
from oslo_utils.secretutils import md5
import yaml

from tempest.lib import auth
from tempest.lib.common import cred_provider
from tempest.lib.common import fixed_network
from tempest.lib import exceptions as lib_exc
from tempest.lib.services import clients

LOG = logging.getLogger(__name__)


def read_accounts_yaml(path):
    try:
        with open(path, 'r') as yaml_file:
            accounts = yaml.safe_load(yaml_file)
    except IOError:
        raise lib_exc.InvalidConfiguration(
            'The path for the test accounts file: %s '
            'could not be found' % path)
    return accounts


class PreProvisionedCredentialProvider(cred_provider.CredentialProvider):
    """Credentials provider using pre-provisioned accounts

    This credentials provider loads the details of pre-provisioned
    accounts from a YAML file, in the format specified by
    ``etc/accounts.yaml.sample``. It locks accounts while in use, using the
    external locking mechanism, allowing for multiple python processes
    to share a single account file, and thus running tests in parallel.

    The accounts_lock_dir must be generated using `lockutils.get_lock_path`
    from the oslo.concurrency library. For instance::

        accounts_lock_dir = os.path.join(lockutils.get_lock_path(CONF),
                                         'test_accounts')

    Role names for object storage are optional as long as the
    `operator` and `reseller_admin` credential types are not used in the
    accounts file.

    :param identity_version: identity version of the credentials
    :param admin_role: name of the admin role
    :param test_accounts_file: path to the accounts YAML file
    :param accounts_lock_dir: the directory for external locking
    :param name: name of the hash file (optional)
    :param credentials_domain: name of the domain credentials belong to
                               (if no domain is configured)
    :param object_storage_operator_role: name of the role
    :param object_storage_reseller_admin_role: name of the role
    :param identity_uri: Identity URI of the target cloud
    """

    # Exclude from the hash fields specific to v2 or v3 identity API
    # i.e. only include user*, project*, tenant* and password
    HASH_CRED_FIELDS = (set(auth.KeystoneV2Credentials.ATTRIBUTES) &
                        set(auth.KeystoneV3Credentials.ATTRIBUTES))

    def __init__(self, identity_version, test_accounts_file,
                 accounts_lock_dir, name=None, credentials_domain=None,
                 admin_role=None, object_storage_operator_role=None,
                 object_storage_reseller_admin_role=None, identity_uri=None):
        super(PreProvisionedCredentialProvider, self).__init__(
            identity_version=identity_version, name=name,
            admin_role=admin_role, credentials_domain=credentials_domain,
            identity_uri=identity_uri)
        self.test_accounts_file = test_accounts_file
        if test_accounts_file:
            accounts = read_accounts_yaml(self.test_accounts_file)
        else:
            raise lib_exc.InvalidCredentials("No accounts file specified")
        self.hash_dict = self.get_hash_dict(
            accounts, admin_role, object_storage_operator_role,
            object_storage_reseller_admin_role)
        self.accounts_dir = accounts_lock_dir
        self._creds = {}

    @classmethod
    def _append_role(cls, role, account_hash, hash_dict):
        if role in hash_dict['roles']:
            hash_dict['roles'][role].append(account_hash)
        else:
            hash_dict['roles'][role] = [account_hash]
        return hash_dict

    @classmethod
    def _append_scoped_role(cls, scope, role, account_hash, hash_dict):
        key = "%s_%s" % (scope, role)
        hash_dict['scoped_roles'].setdefault(key, [])
        hash_dict['scoped_roles'][key].append(account_hash)
        return hash_dict

    @classmethod
    def get_hash_dict(cls, accounts, admin_role,
                      object_storage_operator_role=None,
                      object_storage_reseller_admin_role=None):
        hash_dict = {'roles': {}, 'creds': {}, 'networks': {},
                     'scoped_roles': {}}

        # Loop over the accounts read from the yaml file
        for account in accounts:
            roles = []
            types = []
            scope = None
            resources = []
            if 'roles' in account:
                roles = account.pop('roles')
            if 'types' in account:
                types = account.pop('types')
            if 'resources' in account:
                resources = account.pop('resources')
            if 'project_name' in account:
                scope = 'project'
            elif 'domain_name' in account:
                scope = 'domain'
            elif 'system' in account:
                scope = 'system'
            temp_hash = md5(usedforsecurity=False)
            account_for_hash = dict((k, v) for (k, v) in account.items()
                                    if k in cls.HASH_CRED_FIELDS)
            temp_hash.update(str(account_for_hash).encode('utf-8'))
            temp_hash_key = temp_hash.hexdigest()
            hash_dict['creds'][temp_hash_key] = account
            for role in roles:
                hash_dict = cls._append_role(role, temp_hash_key,
                                             hash_dict)
                if scope:
                    hash_dict = cls._append_scoped_role(
                        scope, role, temp_hash_key, hash_dict)
            # If types are set for the account append the matching role
            # subdict with the hash
            for type in types:
                if type == 'admin':
                    hash_dict = cls._append_role(admin_role, temp_hash_key,
                                                 hash_dict)
                elif type == 'operator':
                    if object_storage_operator_role:
                        hash_dict = cls._append_role(
                            object_storage_operator_role, temp_hash_key,
                            hash_dict)
                    else:
                        msg = ("Type 'operator' configured, but no "
                               "object_storage_operator_role specified")
                        raise lib_exc.InvalidCredentials(msg)
                elif type == 'reseller_admin':
                    if object_storage_reseller_admin_role:
                        hash_dict = cls._append_role(
                            object_storage_reseller_admin_role,
                            temp_hash_key,
                            hash_dict)
                    else:
                        msg = ("Type 'reseller_admin' configured, but no "
                               "object_storage_reseller_admin_role specified")
                        raise lib_exc.InvalidCredentials(msg)
            # Populate the network subdict
            for resource in resources:
                if resource == 'network':
                    hash_dict['networks'][temp_hash_key] = resources[resource]
                else:
                    LOG.warning(
                        'Unknown resource type %s, ignoring this field',
                        resource
                    )
        return hash_dict

    def is_multi_user(self):
        return len(self.hash_dict['creds']) > 1

    def is_multi_tenant(self):
        return self.is_multi_user()

    def _create_hash_file(self, hash_string):
        path = os.path.join(self.accounts_dir, hash_string)
        if not os.path.isfile(path):
            with open(path, 'w') as fd:
                fd.write(self.name)
            return True
        return False

    @lockutils.synchronized('test_accounts_io', external=True)
    def _get_free_hash(self, hashes):
        # Cast as a list because in some edge cases a set will be passed in
        hashes = list(hashes)
        if not os.path.isdir(self.accounts_dir):
            os.mkdir(self.accounts_dir)
            # Create File from first hash (since none are in use)
            self._create_hash_file(hashes[0])
            return hashes[0]
        names = []
        for _hash in hashes:
            res = self._create_hash_file(_hash)
            if res:
                return _hash
            else:
                path = os.path.join(self.accounts_dir, _hash)
                with open(path, 'r') as fd:
                    names.append(fd.read())
        msg = ('Insufficient number of users provided. %s have allocated all '
               'the credentials for this allocation request' % ','.join(names))
        raise lib_exc.InvalidCredentials(msg)

    def _get_match_hash_list(self, roles=None, scope=None):
        hashes = []
        if roles:
            # Loop over all the creds for each role in the subdict and generate
            # a list of cred lists for each role
            for role in roles:
                if scope:
                    key = "%s_%s" % (scope, role)
                    temp_hashes = self.hash_dict['scoped_roles'].get(key)
                    if not temp_hashes:
                        raise lib_exc.InvalidCredentials(
                            "No credentials matching role: %s, scope: %s "
                            "specified in the accounts file" % (role, scope))
                else:
                    temp_hashes = self.hash_dict['roles'].get(role, None)
                    if not temp_hashes:
                        raise lib_exc.InvalidCredentials(
                            "No credentials with role: %s specified in the "
                            "accounts file" % role)
                hashes.append(temp_hashes)
            # Take the list of lists and do a boolean and between each list to
            # find the creds which fall under all the specified roles
            temp_list = set(hashes[0])
            for hash_list in hashes[1:]:
                temp_list = temp_list & set(hash_list)
            hashes = temp_list
        else:
            hashes = self.hash_dict['creds'].keys()
        # NOTE(mtreinish): admin is a special case because of the increased
        # privilege set which could potentially cause issues on tests where
        # that is not expected. So unless the admin role isn't specified do
        # not allocate admin.
        admin_hashes = self.hash_dict['roles'].get(self.admin_role,
                                                   None)
        if ((not roles or self.admin_role not in roles) and
                admin_hashes):
            useable_hashes = [x for x in hashes if x not in admin_hashes]
        else:
            useable_hashes = hashes
        return useable_hashes

    def _sanitize_creds(self, creds):
        temp_creds = creds.copy()
        temp_creds.pop('password')
        return temp_creds

    def _get_creds(self, roles=None, scope=None):
        useable_hashes = self._get_match_hash_list(roles, scope)
        if not useable_hashes:
            msg = 'No users configured for type/roles %s' % roles
            raise lib_exc.InvalidCredentials(msg)
        free_hash = self._get_free_hash(useable_hashes)
        clean_creds = self._sanitize_creds(
            self.hash_dict['creds'][free_hash])
        LOG.info('%s allocated creds:\n%s', self.name, clean_creds)
        return self._wrap_creds_with_network(free_hash)

    @lockutils.synchronized('test_accounts_io', external=True)
    def remove_hash(self, hash_string):
        hash_path = os.path.join(self.accounts_dir, hash_string)
        if not os.path.isfile(hash_path):
            LOG.warning('Expected an account lock file %s to remove, but '
                        'one did not exist', hash_path)
        else:
            os.remove(hash_path)
            if not os.listdir(self.accounts_dir):
                os.rmdir(self.accounts_dir)

    def get_hash(self, creds):
        for _hash in self.hash_dict['creds']:
            # Comparing on the attributes that are expected in the YAML
            init_attributes = creds.get_init_attributes()
            # Only use the attributes initially used to calculate the hash
            init_attributes = [x for x in init_attributes if
                               x in self.HASH_CRED_FIELDS]
            hash_attributes = self.hash_dict['creds'][_hash].copy()
            # NOTE(andreaf) Not all fields may be available on all credentials
            # so defaulting to None for that case.
            if all([getattr(creds, k, None) == hash_attributes.get(k, None) for
                    k in init_attributes]):
                return _hash
        raise AttributeError('Invalid credentials %s' % creds)

    def remove_credentials(self, creds):
        _hash = self.get_hash(creds)
        clean_creds = self._sanitize_creds(self.hash_dict['creds'][_hash])
        self.remove_hash(_hash)
        LOG.info("%s returned allocated creds:\n%s", self.name, clean_creds)

    # TODO(gmann): Remove this method in favor of get_project_member_creds()
    # after the deprecation phase.
    def get_primary_creds(self):
        if self._creds.get('primary'):
            return self._creds.get('primary')
        net_creds = self._get_creds()
        self._creds['primary'] = net_creds
        return net_creds

    # TODO(gmann): Replace this method with more appropriate name.
    # like get_project_alt_member_creds()
    def get_alt_creds(self):
        if self._creds.get('alt'):
            return self._creds.get('alt')
        net_creds = self._get_creds()
        self._creds['alt'] = net_creds
        return net_creds

    def get_system_admin_creds(self):
        if self._creds.get('system_admin'):
            return self._creds.get('system_admin')
        system_admin = self._get_creds(['admin'], scope='system')
        self._creds['system_admin'] = system_admin
        return system_admin

    def get_system_member_creds(self):
        if self._creds.get('system_member'):
            return self._creds.get('system_member')
        system_member = self._get_creds(['member'], scope='system')
        self._creds['system_member'] = system_member
        return system_member

    def get_system_reader_creds(self):
        if self._creds.get('system_reader'):
            return self._creds.get('system_reader')
        system_reader = self._get_creds(['reader'], scope='system')
        self._creds['system_reader'] = system_reader
        return system_reader

    def get_domain_admin_creds(self):
        if self._creds.get('domain_admin'):
            return self._creds.get('domain_admin')
        domain_admin = self._get_creds(['admin'], scope='domain')
        self._creds['domain_admin'] = domain_admin
        return domain_admin

    def get_domain_manager_creds(self):
        if self._creds.get('domain_manager'):
            return self._creds.get('domain_manager')
        domain_manager = self._get_creds(['manager'], scope='domain')
        self._creds['domain_manager'] = domain_manager
        return domain_manager

    def get_domain_member_creds(self):
        if self._creds.get('domain_member'):
            return self._creds.get('domain_member')
        domain_member = self._get_creds(['member'], scope='domain')
        self._creds['domain_member'] = domain_member
        return domain_member

    def get_domain_reader_creds(self):
        if self._creds.get('domain_reader'):
            return self._creds.get('domain_reader')
        domain_reader = self._get_creds(['reader'], scope='domain')
        self._creds['domain_reader'] = domain_reader
        return domain_reader

    def get_project_admin_creds(self):
        if self._creds.get('project_admin'):
            return self._creds.get('project_admin')
        project_admin = self._get_creds(['admin'], scope='project')
        self._creds['project_admin'] = project_admin
        return project_admin

    def get_project_alt_admin_creds(self):
        # TODO(gmann): Implement alt admin hash.
        return

    def get_project_manager_creds(self):
        if self._creds.get('project_manager'):
            return self._creds.get('project_manager')
        project_manager = self._get_creds(['manager'], scope='project')
        self._creds['project_manager'] = project_manager
        return project_manager

    def get_project_member_creds(self):
        if self._creds.get('project_member'):
            return self._creds.get('project_member')
        project_member = self._get_creds(['member'], scope='project')
        self._creds['project_member'] = project_member
        return project_member

    def get_project_alt_member_creds(self):
        # TODO(gmann): Implement alt member hash.
        return

    def get_project_reader_creds(self):
        if self._creds.get('project_reader'):
            return self._creds.get('project_reader')
        project_reader = self._get_creds(['reader'], scope='project')
        self._creds['project_reader'] = project_reader
        return project_reader

    def get_project_alt_reader_creds(self):
        # TODO(gmann): Implement alt reader hash.
        return

    def get_creds_by_roles(self, roles, force_new=False, scope=None):
        roles = list(set(roles))
        exist_creds = self._creds.get(str(roles).encode(
            'utf-8'), None)
        # The force kwarg is used to allocate an additional set of creds with
        # the same role list. The index used for the previously allocation
        # in the _creds dict will be moved.
        if exist_creds and not force_new:
            return exist_creds
        elif exist_creds and force_new:
            # NOTE(andreaf) In py3.x encode returns bytes, and b'' is bytes
            # In py2.7 encode returns strings, and b'' is still string
            new_index = str(roles).encode('utf-8') + b'-' + \
                str(len(self._creds)).encode('utf-8')
            self._creds[new_index] = exist_creds
        net_creds = self._get_creds(roles=roles)
        self._creds[str(roles).encode('utf-8')] = net_creds
        return net_creds

    def clear_creds(self):
        for creds in self._creds.values():
            self.remove_credentials(creds)

    # TODO(gmann): Remove this method in favor of get_project_admin_creds()
    # after the deprecation phase.
    def get_admin_creds(self):
        return self.get_creds_by_roles([self.admin_role])

    def is_role_available(self, role):
        if self.hash_dict['roles'].get(role):
            return True
        return False

    def admin_available(self):
        return self.is_role_available(self.admin_role)

    def _wrap_creds_with_network(self, hash):
        creds_dict = self.hash_dict['creds'][hash]
        # Make sure a domain scope if defined for users in case of V3
        # Make sure a tenant is available in case of V2
        creds_dict = self._extend_credentials(creds_dict)
        # This just builds a Credentials object, it does not validate
        # nor fill  with missing fields.
        credential = auth.get_credentials(
            auth_url=None, fill_in=False,
            identity_version=self.identity_version, **creds_dict)
        net_creds = cred_provider.TestResources(credential)
        net_clients = clients.ServiceClients(credentials=credential,
                                             identity_uri=self.identity_uri)
        networks_client = net_clients.network.NetworksClient()
        net_name = self.hash_dict['networks'].get(hash, None)
        try:
            network = fixed_network.get_network_from_name(
                net_name, networks_client)
        except lib_exc.InvalidTestResource:
            network = {}
        net_creds.set_resources(network=network)
        return net_creds

    def _extend_credentials(self, creds_dict):
        # Add or remove credential domain fields to fit the identity version
        domain_fields = set(x for x in auth.KeystoneV3Credentials.ATTRIBUTES
                            if 'domain' in x)
        msg = 'Assuming they are valid in the default domain.'
        if self.identity_version == 'v3':
            if not domain_fields.intersection(set(creds_dict.keys())):
                msg = 'Using credentials %s for v3 API calls. ' + msg
                LOG.warning(msg, self._sanitize_creds(creds_dict))
                creds_dict['domain_name'] = self.credentials_domain
        if self.identity_version == 'v2':
            if domain_fields.intersection(set(creds_dict.keys())):
                msg = 'Using credentials %s for v2 API calls. ' + msg
                LOG.warning(msg, self._sanitize_creds(creds_dict))
            # Remove all valid domain attributes
            for attr in domain_fields.intersection(set(creds_dict.keys())):
                creds_dict.pop(attr)
        return creds_dict
